@@ -9,7 +9,7 @@
 
 package org.gnu.atk;
 
-import org.gnu.javagnome.Enum;
+import org.gnu.glib.Enum;
 
 
 /**
