@@ -11,7 +11,7 @@ package org.gnu.gdk;
 
 import org.gnu.glib.GObject;
 import org.gnu.glib.Type;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * Display objects are the GDK representation of the X Display which
