@@ -9,7 +9,7 @@
 
 package org.gnu.gdk;
 
-import org.gnu.javagnome.Enum;
+import org.gnu.glib.Enum;
 
 public class InterpType extends Enum 
 {
