@@ -9,14 +9,13 @@
 
 package org.gnu.gdk;
 
-import org.gnu.glib.Boxed;
 import org.gnu.glib.Handle;
+import org.gnu.glib.MemStruct;
 
-public class RgbCmap extends Boxed 
+public class RgbCmap extends MemStruct
 {
 
     native static final protected Handle gdk_rgb_cmap_new (int [] colors, int numColors);
-    native static final protected void gdk_rgb_cmap_free (Handle cmap);
 
 }
 
