@@ -10,7 +10,7 @@
 package org.gnu.glib;
 
 import java.util.Vector;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * @deprecated This class is deprecated. Use Java's collection classes
