@@ -11,10 +11,9 @@ package org.gnu.glib;
 import java.util.ArrayList;
 import java.util.Hashtable;
 import java.util.ListIterator;
+//import java.util.logging.Logger;
 
 import org.gnu.gdk.Pixbuf;
-import org.gnu.javagnome.Handle;
-import org.gnu.javagnome.Struct;
 
 /**
  * Base class for all objects that participate in the GLib
@@ -22,6 +21,8 @@ import org.gnu.javagnome.Struct;
  */
 public class GObject extends Struct {
 
+    //private static final Logger log = Logger.getLogger("org.gnu.glib.GObject");
+    
     /** Event initialization marker. */
     protected boolean eventsInitialized = false;
     /** Java-side properties container. */
@@ -44,7 +45,7 @@ public class GObject extends Struct {
      * Create a new GObject of the given type.
      */
     public GObject( Type type ) {
-        setHandle( g_object_new( type.getTypeHandle() ) );
+        super( g_object_new( type.getTypeHandle() ) );
     }
 
     /**
@@ -54,7 +55,7 @@ public class GObject extends Struct {
      * @param handle The handle that represents a pointer to a native resource.
      */
     public GObject(Handle handle) {
-        setHandle(handle);
+        super(handle);
     }
 	
     /** 
@@ -175,13 +176,6 @@ public class GObject extends Struct {
         }
         setData(getHandle(), key, data);
     }
-	
-    /**
-     * Sets this object's native handle.
-     */
-    protected void setHandle(Handle hndl) {
-        super.setHandle(hndl);
-    }
 
     public Class getEventListenerClass(String signal) {
         return null;
@@ -599,8 +593,19 @@ public class GObject extends Struct {
     }
     
     protected void finalize() throws Throwable {
-        nativeFinalize(getHandle());
-        super.finalize();
+        //log.fine(getClass().getName());
+        
+        // If an exception is thrown in the constructor of a subclass, then
+        // an object with a null handle may be finalized at some point. Don't
+        // run nativeFinalize for those objects.
+        try {
+            if (getHandle() != null) {
+                nativeFinalize(getHandle());
+            }
+        }
+        finally {
+            super.finalize();
+        }
     }
 
     // 
@@ -608,8 +613,6 @@ public class GObject extends Struct {
     //
 
     private static void printStackTrace(String message) {
-        //System.out.println("Printing stacktrace for native error!");
-        //System.out.println("Message was: " + message);
         (new Exception(message)).printStackTrace();
     }
 
@@ -717,9 +720,9 @@ public class GObject extends Struct {
     // ------------------------------------------------------------------------
     // static init code
     static {
-        System.loadLibrary(org.gnu.glib.Config.LIBRARY_NAME + org.gnu.glib.Config.GTK_API_VERSION);
-        init();
+        System.loadLibrary(org.gnu.gtk.Config.LIBRARY_NAME + org.gnu.gtk.Config.GTK_API_VERSION);
         registerLogHandler();
+        init();
     }
 
 }
