@@ -9,8 +9,8 @@
 
 package org.gnu.glib;
 
-import org.gnu.javagnome.Handle;
-import org.gnu.javagnome.Struct;
+import org.gnu.glib.Handle;
+import org.gnu.glib.Struct;
 
 /**
  * Class for memory management of structs.  Should be renamed to
@@ -29,10 +29,26 @@ public class MemStruct extends Struct
     }
 
     protected void finalize() throws Throwable {
-        nativeFinalize(getHandle());
+        //log.fine(this.getClass().getName());
+    	
+        // If an exception is throws in the constructor of a sublcass, then
+        // an object with a null handle may be finalized at some point. Don't
+        // run nativeFinalize for those objects.
+        try {
+            if (getHandle() != null) {
+                nativeFinalize(getHandle());
+            }
+        }
+        finally {
+            super.finalize();
+        }
     }
 
+    static {
+        init();
+    }
+    
     native static final private void nativeFinalize(Handle handle);
-
+    native static final private void init();
 }
 
