@@ -11,7 +11,7 @@ package org.gnu.gtk;
 
 import org.gnu.atk.AtkObject;
 import org.gnu.glib.Type;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  *
@@ -22,6 +22,20 @@ public class Accessible extends AtkObject
 	public Accessible(Handle handle) {
 		super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static Accessible getAccessible(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Accessible obj = (Accessible) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new Accessible(handle);
+        
+        return obj;
+    }
 	
 	public Widget getWidget() {
 		Handle hndl = getWidget(getHandle());
