@@ -17,7 +17,7 @@ import org.gnu.glib.GObject;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.AdjustmentEvent;
 import org.gnu.gtk.event.AdjustmentListener;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * This object stores geometry information, such as upper and lower
@@ -46,7 +46,10 @@ public class Adjustment extends GtkObject {
 		super(handle);
 	}
     
-    static Adjustment getAdjustment(Handle handle) {
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static Adjustment getAdjustment(Handle handle) {
         if (handle == null)
             return null;
         
