@@ -10,7 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Type;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * This container widget wraps a single widget in a frame with a label,
@@ -43,6 +43,20 @@ public class AspectFrame extends Frame {
 	public AspectFrame(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static AspectFrame getAspectFrame(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        AspectFrame obj = (AspectFrame) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new AspectFrame(handle);
+        
+        return obj;
+    }
 
 	/**
 	 * Set the aspect for an already created AspectFrame.
