@@ -10,7 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Type;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * This is the base class of HBox and VBox. It has the code
@@ -32,6 +32,20 @@ public class Box extends Container
 	protected Box(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    protected static Box getBox(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Box obj = (Box) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new Box(handle);
+        
+        return obj;
+    }
 
 	/**
 	 * Adds child to the box, packed with reference to the start of box. 
