@@ -13,9 +13,9 @@ import java.util.Vector;
 
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
-import org.gnu.glib.Handle;
 import org.gnu.gtk.event.ButtonEvent;
 import org.gnu.gtk.event.ButtonListener;
+import org.gnu.glib.Handle;
 
 /**
  * A widget that creates a signal when clicked on. This is generally 
@@ -89,6 +89,20 @@ public class Button extends Bin {
 	public Button(Handle handle) {
 		super(handle);
 	}
+    
+    /**
+     * Construct a button using a handle to a native resource.
+     */
+    public static Button getButton(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Button obj = (Button) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new Button(handle);
+        
+        return obj;
+    }
 
 	/**
 	 * Sends a <i>pressed</i> signal to the button. If handlers have been set 
@@ -176,8 +190,7 @@ public class Button extends Bin {
 		return gtk_button_get_use_underline(getHandle());
 	}
 
-	// TODO: give a doc link to list of stock items. an example application,
-	// perhaps.
+	// TODO: give a doc link to list of stock items.
 	/**
 	 * Setting this will true will make the widget try to interpret the label as
 	 * a stock item. If the stock id is unknown, it will be treated as a normal
@@ -229,7 +242,7 @@ public class Button extends Bin {
 	}
 	
 	public Widget getImage() {
-		return new Widget(gtk_button_get_image(getHandle()));
+		return Widget.getWidget(gtk_button_get_image(getHandle()));
 	}
 	
 	
