@@ -14,10 +14,10 @@ import java.util.Vector;
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.DialogEvent;
 import org.gnu.gtk.event.DialogListener;
+import org.gnu.glib.Handle;
 
 /**
  * Dialog boxes are a convenient way to prompt the user for a small amount of 
@@ -55,6 +55,23 @@ public class Dialog extends Window {
 	public Dialog(Handle handle) {
 		super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static Dialog getDialog(Handle handle) {
+        if (handle == null) {
+            return null;
+        }
+        
+        Dialog obj = (Dialog) GObject.getGObjectFromHandle(handle);
+        
+        if (obj == null) {
+            obj = new Dialog(handle);
+        }
+        
+        return obj;
+    }
 
 	/**
 	 * Blocks in a recursive main loop until the <i>dialog</i> either emits a
@@ -62,32 +79,43 @@ public class Dialog extends Window {
 	 * a {@link ResponseType} to determine if a standard return value is emitted.
 	 * The developer can also set the response type when a button or widget is
 	 * added to the dialog.  If the dialog is destroyed the run method will 
-	 * return RESPONSE_NONE.
+	 * return {@link ResponseType#NONE}.
 	 * 
 	 * @return The response that indicates which button the user pressed or
-	 * RESPONSE_NONE if the window was destroyed.
+	 * {@link ResponseType#NONE} if the window was destroyed.
 	 */
 	public int run() {
+	    checkState();
 		return Dialog.gtk_dialog_run(getHandle());
 	}
 
 	/**
-	 * Adds a button with the given text (or a stock button if buttonText is 
-	 * a stock ID) and sets things up so that clicking the button will emit
-	 * the "response" event with the given <code>responseId</code>.
+	 * Adds a button with the given text and sets things up so that 
+	 * clicking the button will emit the "response" event with the 
+	 * given <code>responseId</code>.
 	 * 
-	 * An example of stock ID: "gtk-ok" creates an OK button. 
-	 * You may see this <a href="http://developer.gnome.org/doc/API/2.0/gtk/gtk-Stock-Items.html">list of
-	 * stock IDs</a> avaliable in the Gtk toolkit. 
-	 * 
-	 * @param buttonText The text string to display on the button or the
-	 * stock ID for the button.
+	 * @param buttonText The text string to display on the button.
 	 * @param responseId The response ID to emit when the user clicks the
 	 * button that is to be added.
 	 */
 	public void addButton(String buttonText, int responseId) {
+        checkState();
 		Dialog.gtk_dialog_add_button(getHandle(), buttonText, responseId);
 	}
+	
+	/**
+	 * Adds a button with the given stock button and sets things up so 
+	 * that clicking the button will emit the "response" event with 
+	 * the given <code>responseId</code>.
+	 * 
+	 * @param stockItem The stock ID for the button to be added.
+	 * @param responseId The response ID to emit when the user clicks the
+	 * button that is to be added.
+	 */
+	public void addButton(GtkStockItem stockItem, int responseId) {
+        checkState();
+		addButton(stockItem.getString(), responseId);
+	}
 
 	/**
 	 * Adds an activatable widget to the action area of the Dialog, connecting
@@ -100,6 +128,7 @@ public class Dialog extends Window {
 	 * widget.
 	 */
 	public void addWidget(Widget child, int responseId) {
+        checkState();
 		Dialog.gtk_dialog_add_action_widget(getHandle(), child.getHandle(), responseId);
 	}
 
@@ -111,6 +140,7 @@ public class Dialog extends Window {
 	 * @param responseId The response ID.
 	 */
 	public void setDefaultResponse(int responseId) {
+        checkState();
 		Dialog.gtk_dialog_set_default_response(getHandle(), responseId);
 	}
 
@@ -121,25 +151,21 @@ public class Dialog extends Window {
 	 * @return The VBox that is the layout control for the dialog.
 	 */
 	public VBox getDialogLayout() {
+        checkState();
 	    Handle hndl = getVbox(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (VBox)obj;
-		return new VBox(hndl);
+		return VBox.getVBox(hndl);
 	}
 
 	/**
 	 * The lower area of the Dialog where the buttons are located is a HButtonBox.
-	 * This methos will return that container.
+	 * This method will return that container.
 	 * 
 	 * @return The HButton box that is known as the action area.
 	 */
 	public HButtonBox getActionArea() {
+        checkState();
 	    Handle hndl = Dialog.getActionArea(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj) 
-			return (HButtonBox)obj;
-		return new HButtonBox(hndl);
+		return HButtonBox.getHButtonBox(hndl);
 	}
 
 	/**
@@ -149,6 +175,7 @@ public class Dialog extends Window {
 	 * @param setting Indicates if a separator should be added to the dialog.
 	 */
 	public void setHasSeparator(boolean setting) {
+        checkState();
 		Dialog.gtk_dialog_set_has_separator(getHandle(), setting);
 	}
 
@@ -158,9 +185,23 @@ public class Dialog extends Window {
 	 * @return true if the Dialog has a separator.
 	 */
 	public boolean getHasSeparator() {
+        checkState();
 		return Dialog.gtk_dialog_get_has_separator(getHandle());
 	}
 
+	/**
+	 * Gets the response id of a widget in the action area
+	 * of a dialog. 
+	 * @param widget a widget in the action area of this Dialog.
+	 * @return the response id of the widget, or ResponseType.NONE
+	 * if the widget doesn't have a response id set.
+	 */
+	public ResponseType getResponseForWidget(Widget widget) {
+        checkState();
+		return ResponseType.intern(
+				gtk_dialog_get_response_for_widget(getHandle(), widget.getHandle()));
+	}
+	
 	/* **************************************
 	 * EVENT LISTENERS
 	 ****************************************/
@@ -175,6 +216,7 @@ public class Dialog extends Window {
 	 * @see org.gnu.gtk.event.DialogListener
 	 */
 	public void addListener(DialogListener listener) {
+        checkState();
 		// Don't add the listener a second time if it is in the Vector.
 		int i = findListener(dialogListeners, listener);
 		if (i == -1) {
@@ -191,6 +233,7 @@ public class Dialog extends Window {
 	 * @see #addListener(org.gnu.gtk.event.DialogListener)
 	 */
 	public void removeListener(DialogListener listener) {
+        checkState();
 		int i = findListener(dialogListeners, listener);
 		if (i > -1) {
 			dialogListeners.remove(i);
@@ -226,12 +269,14 @@ public class Dialog extends Window {
 	}
 
 	public Class getEventListenerClass(String signal) {
+        checkState();
 		Class cls = evtMap.getEventListenerClass(signal);
 		if (cls == null) cls = super.getEventListenerClass(signal);
 		return cls;
 	}
 
 	public EventType getEventType(String signal) {
+        checkState();
 		EventType et = evtMap.getEventType(signal);
 		if (et == null) et = super.getEventType(signal);
 		return et;
@@ -271,5 +316,7 @@ public class Dialog extends Window {
 	native static final protected boolean gtk_dialog_get_has_separator(Handle dialog);
 	native static final protected void gtk_dialog_response(Handle dialog, int responseId);
 	native static final protected int gtk_dialog_run(Handle dialog);
+	// new for gtk 2.8
+	native static final protected int gtk_dialog_get_response_for_widget(Handle dialog, Handle widget);
 
 }
