@@ -9,8 +9,9 @@
 
 package org.gnu.gtk;
 
+import org.gnu.glib.GObject;
 import org.gnu.glib.Type;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * The DrawingArea widget is used for creating custom user interface elements.
@@ -28,6 +29,23 @@ public class DrawingArea extends Widget
 	public DrawingArea(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static DrawingArea getDrawingArea(Handle handle) {
+        if (handle == null) {
+            return null;
+        }
+        
+        DrawingArea obj = (DrawingArea) GObject.getGObjectFromHandle(handle);
+        
+        if (obj == null) {
+            obj = new DrawingArea(handle);
+        }
+        
+        return obj;
+    }
 
 	/**
 	 * Retrieve the runtime type used by the GLib library.
