@@ -18,7 +18,7 @@ import org.gnu.glib.GObject;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.EntryEvent;
 import org.gnu.gtk.event.EntryListener;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * A single line text entry widget. A fairly large set of key bindings are 
@@ -39,6 +39,9 @@ public class Entry extends Widget implements Editable {
 	    super(handle);
 	}
     
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
     public static Entry getEntry(Handle handle) {
         if (handle == null)
             return null;
