@@ -13,9 +13,9 @@ import java.util.Vector;
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.gtk.event.EntryCompletionEvent;
 import org.gnu.gtk.event.EntryCompletionListener;
+import org.gnu.glib.Handle;
 
 /**
  * EntryCompletion is an auxiliary object to be used in conjunction 
@@ -40,16 +40,27 @@ public class EntryCompletion extends GObject {
 	public EntryCompletion(Handle nativeHandle) {
 		super(nativeHandle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static EntryCompletion getEntryCompletion(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        EntryCompletion obj = (EntryCompletion) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new EntryCompletion(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Returns the Entry this completion is attached to.
 	 */
 	public Entry getEntry() {
 	    Handle hndl = gtk_entry_completion_get_entry(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (Entry)obj;
-		return new Entry(hndl);
+		return Entry.getEntry(hndl);
 	}
 	
 	/**
@@ -67,10 +78,7 @@ public class EntryCompletion extends GObject {
 	 */
 	public TreeModel getModel() {
 	    Handle hndl = gtk_entry_completion_get_model(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (TreeModel)obj;
-		return new TreeModel(hndl);
+		return TreeModel.getTreeModel(hndl);
 	}
 	
 	/**
@@ -184,8 +192,17 @@ public class EntryCompletion extends GObject {
 	public int getTextColumn() {
 		return gtk_entry_completion_get_text_column(getHandle());
 	}
+
+	public void setPopupSetWidth(boolean setWidth) {
+		gtk_entry_completion_set_popup_set_width(getHandle(), setWidth);
+	}
 	
-	/* **************************************
+	public boolean getPopupSetWidth() {
+		return gtk_entry_completion_get_popup_set_width(getHandle());
+	}
+
+	
+	/*
 	 * EVENT LISTENERS
 	 ****************************************/
 
@@ -242,14 +259,9 @@ public class EntryCompletion extends GObject {
 
 	private boolean handleMatchSelected(Handle model, Handle iter) {
 		EntryCompletionEvent evt = new EntryCompletionEvent(this, EntryCompletionEvent.Type.MATCH_SELECTED);
-		TreeModelFilter tm;
-		GObject obj = getGObjectFromHandle(model);
-		if (null != obj)
-			tm = (TreeModelFilter)obj;
-		else
-			tm = new TreeModelFilter(model);
+		TreeModelFilter tm = TreeModelFilter.getTreeModelFilter(model);
 		evt.setModel(tm);
-		evt.setIter(new TreeIter(iter, tm));
+		evt.setIter(TreeIter.getTreeIter(iter, tm));
 		fireEntrySelectionEvent(evt);
 		return true;
 	}
@@ -318,5 +330,8 @@ public class EntryCompletion extends GObject {
 	native static final protected boolean gtk_entry_completion_get_popup_completion(Handle completion);
 	native static final protected void gtk_entry_completion_set_text_column(Handle completion,  int column);
 	native static final protected int gtk_entry_completion_get_text_column(Handle completion);
+	// New in 2.8
+	native static final protected void gtk_entry_completion_set_popup_set_width(Handle completion, boolean popupSetWidth);
+	native static final protected boolean gtk_entry_completion_get_popup_set_width(Handle completion);
 
 }
