@@ -9,22 +9,62 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.GObject;
+import java.util.ArrayList;
+import java.util.List;
+
+import org.gnu.glib.EventMap;
 import org.gnu.glib.Handle;
+import org.gnu.gtk.event.FileChooserEvent;
+import org.gnu.gtk.event.FileChooserListener;
 
 /**
  * 
  */
 public class FileChooserDialog extends Dialog implements FileChooser {
 	
+    private List listeners;
+    private static EventMap eventMap = new EventMap();
+    
+    static {
+        addEvents(eventMap);
+    }
+    
 	public FileChooserDialog(Handle handle) {
 		super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static FileChooserDialog getFileChooserDialog(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        FileChooserDialog obj = (FileChooserDialog) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new FileChooserDialog(handle);
+        
+        return obj;
+    }
 
 	public FileChooserDialog(String title, Window parent, FileChooserAction action) {
 		super(init(title, parent, action));
 	}
 	
+    /**
+     * Works similarly to {@link Dialog#run()}. The only difference is that it
+     * also adds the {@link GtkStockItem#CANCEL} and the {@link GtkStockItem#OK}
+     * buttons to the FileChooserDialog.
+     * 
+     * @return The response that indicates which button the user pressed or
+     * {@link ResponseType#NONE} if the window was destroyed.
+     */
+    public int open() {
+        addButton(GtkStockItem.CANCEL, ResponseType.CANCEL.getValue());
+        addButton(GtkStockItem.OK, ResponseType.OK.getValue());
+        return run();
+    }
+    
 	private static Handle init(String title, Window parent, 
                                    FileChooserAction action) {
 	    Handle hndl = getNullHandle();
@@ -93,7 +133,7 @@ public class FileChooserDialog extends Dialog implements FileChooser {
 	 * @see org.gnu.gtk.FileChooser#setFilename(java.lang.String)
 	 */
 	public boolean setFilename(String filename) {
-		return FileChooserHelper.gtk_file_chooser_set_filename(getHandle(), filename);
+		return FileChooserHelper.setFilename(getHandle(), filename);
 	}
 
 	/* (non-Javadoc)
@@ -206,12 +246,7 @@ public class FileChooserDialog extends Dialog implements FileChooser {
 	 */
 	public Widget getPreviewWidget() {
 	    Handle hndl = FileChooserHelper.gtk_file_chooser_get_preview_widget(getHandle());
-		if (null == hndl)
-			return null;
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (Widget)obj;
-		return new Widget(hndl);
+		return Widget.getWidget(hndl);
 	}
 
 	/* (non-Javadoc)
@@ -268,12 +303,7 @@ public class FileChooserDialog extends Dialog implements FileChooser {
 	 */
 	public Widget getExtraWidget() {
 	    Handle hndl = FileChooserHelper.gtk_file_chooser_get_extra_widget(getHandle());
-		if (null == hndl)
-			return null;
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (Widget)obj;
-		return new Widget(hndl);
+		return Widget.getWidget(hndl);
 	}
 
 	/* (non-Javadoc)
@@ -297,11 +327,7 @@ public class FileChooserDialog extends Dialog implements FileChooser {
 	    Handle[] hndls = FileChooserHelper.gtk_file_chooser_list_filters(getHandle());
 		FileFilter[] filters = new FileFilter[hndls.length];
 		for (int i = 0; i < hndls.length; i++) {
-			GObject obj = getGObjectFromHandle(hndls[i]);
-			if (null != obj)
-				filters[i] = (FileFilter)obj;
-			else
-				filters[i] = new FileFilter(hndls[i]);
+			filters[i] = FileFilter.getFileFilter(hndls[i]);
 		}
 		return filters;
 	}
@@ -318,30 +344,21 @@ public class FileChooserDialog extends Dialog implements FileChooser {
 	 */
 	public FileFilter getFilter() {
 	    Handle hndl = FileChooserHelper.gtk_file_chooser_get_filter(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (FileFilter)obj;
-		return new FileFilter(hndl);
+		return FileFilter.getFileFilter(hndl);
 	}
 
 	/* (non-Javadoc)
 	 * @see org.gnu.gtk.FileChooser#addShortcutFolder(java.lang.String)
 	 */
 	public void addShortcutFolder(String folder) throws FileChooserException {
-		int err = -1;
-		FileChooserHelper.gtk_file_chooser_add_shortcut_folder(getHandle(), folder, new int[] {err});
-		if (-1 < err)
-			throw new FileChooserException(err);
+        FileChooserHelper.addShortcutFolder(getHandle(), folder);
 	}
 
 	/* (non-Javadoc)
 	 * @see org.gnu.gtk.FileChooser#removeShortcutFolder(java.lang.String)
 	 */
 	public void removeShortcutFolder(String folder) throws FileChooserException {
-		int err = -1;
-		FileChooserHelper.gtk_file_chooser_remove_shortcut_folder(getHandle(), folder, new int[] {err});
-		if (-1 < err)
-			throw new FileChooserException(err);
+        FileChooserHelper.removeShortcutFolder(getHandle(), folder);
 	}
 
 	/* (non-Javadoc)
@@ -355,20 +372,14 @@ public class FileChooserDialog extends Dialog implements FileChooser {
 	 * @see org.gnu.gtk.FileChooser#addShortCutURI(java.lang.String)
 	 */
 	public void addShortcutURI(String uri) throws FileChooserException {
-		int err = -1;
-		FileChooserHelper.gtk_file_chooser_add_shortcut_folder_uri(getHandle(), uri, new int[] {err});
-		if (-1 < err)
-			throw new FileChooserException(err);
+        FileChooserHelper.addShortcutURI(getHandle(), uri);
 	}
 
 	/* (non-Javadoc)
 	 * @see org.gnu.gtk.FileChooser#removeShortcutURI(java.lang.String)
 	 */
 	public void removeShortcutURI(String uri) throws FileChooserException {
-		int err = -1;
-		FileChooserHelper.gtk_file_chooser_remove_shortcut_folder_uri(getHandle(), uri, new int[] {err});
-		if (-1 < err)
-			throw new FileChooserException(err);
+        FileChooserHelper.removeShortcutURI(getHandle(), uri);
 	}
 
 	/* (non-Javadoc)
@@ -397,9 +408,55 @@ public class FileChooserDialog extends Dialog implements FileChooser {
      */
     public boolean getShowHidden() {
         return FileChooserHelper.gtk_file_chooser_get_show_hidden( getHandle() );
+    } 
+    
+    public void addListener(FileChooserListener listener) {
+        if (listeners == null) {
+            listeners = new ArrayList();
+        }
+        FileChooserHelper.addListener(listeners, listener, eventMap, this);
     }
     
+    public void removeListener(FileChooserListener listener) {
+        FileChooserHelper.removeListener(listeners, listener, eventMap, this);
+        if (listeners.size() == 0) {
+            listeners = null;
+        }
+    }
 
+    private static void addEvents(EventMap eventMap) {
+        FileChooserEvent.Type[] types = FileChooserEvent.Type.getTypes();
+        for (int i = 0; i < types.length; ++i) {
+            eventMap.addEvent(types[i], FileChooserDialog.class);
+        }
+    }
+    
+    private void handleCurrentFolderChanged() {
+        FileChooserEvent event = new FileChooserEvent(this,
+                FileChooserEvent.Type.CURRENT_FOLDER_CHANGED);
+        FileChooserHelper.fireCurrentFolderChanged(listeners, event);
+    }
+    
+    private void handleFileActivated() {
+        FileChooserEvent event = new FileChooserEvent(this,
+                FileChooserEvent.Type.CURRENT_FOLDER_CHANGED);
+
+        FileChooserHelper.fireFileActivated(listeners, event);
+    }
+    
+    private void handleSelectionChanged() {
+        FileChooserEvent event = new FileChooserEvent(this,
+                FileChooserEvent.Type.CURRENT_FOLDER_CHANGED);
+        FileChooserHelper.fireSelectionChanged(listeners, event);
+    }
+    
+    private void handleUpdatePreview() {
+        FileChooserEvent event = new FileChooserEvent(this,
+                FileChooserEvent.Type.CURRENT_FOLDER_CHANGED);
+        FileChooserHelper.fireUpdatePreview(listeners, event);
+    }
+    
 	native static final protected int gtk_file_chooser_dialog_get_type ();
-	native static final protected Handle gtk_file_chooser_dialog_new(String title, Handle parent, int action);
+	native static final protected Handle gtk_file_chooser_dialog_new(String title,
+            Handle parent, int action);
 }
