@@ -10,6 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Type;
+import org.gnu.glib.Handle;
 
 /**
  * HButtonBox is a container designed to display a collection of
@@ -32,9 +33,23 @@ public class HButtonBox extends ButtonBox {
 	/**
 	 * Construct a new HButtonBox from a handle to a native resource.
 	 */
-	public HButtonBox(int handle) {
+	public HButtonBox(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Construct a new HButtonBox from a handle to a native resource.
+     */
+    public static HButtonBox getHButtonBox(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        HButtonBox obj = (HButtonBox) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new HButtonBox(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Retrieve the runtime type used by the GLib library.
@@ -43,13 +58,15 @@ public class HButtonBox extends ButtonBox {
 		return new Type(gtk_hbutton_box_get_type());
 	}
 
-    /****************************************
-     * BEGINNING OF GENERATED CODE
-     ****************************************/
+
     native static final protected int gtk_hbutton_box_get_type ();
-    native static final protected int gtk_hbutton_box_new ();
-    /****************************************
-     * END OF GENERATED CODE
-     ****************************************/
+    native static final protected Handle gtk_hbutton_box_new ();
+
+    /* Deprectated functions.
+    native static final private int gtk_hbutton_box_get_spacing_default();
+    native static final private int gtk_hbutton_box_get_layout_default();
+    native static final private void gtk_hbutton_box_set_spacing_default(int spacing);
+    native static final private void gtk_hbutton_box_set_layout_default(int layout);
+    */
 }
 
