@@ -10,7 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Type;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * HRuler displays a horizontally-oriented scale that optionally can
@@ -29,6 +29,20 @@ public class HRuler extends Ruler
 	public HRuler(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static HRuler getHRuler(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        HRuler obj = (HRuler) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new HRuler(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Retrieve the runtime type used by the GLib library.
