@@ -9,8 +9,9 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Handle;
+import org.gnu.glib.GObject;
 import org.gnu.glib.Type;
+import org.gnu.glib.Handle;
 
 /**
  * The HScrollBar displays a horizontal scrollbar with a slider and
@@ -24,11 +25,11 @@ public class HScrollBar extends ScrollBar
 {
 	/**
 	 * Creates a new ScrollBar
-	 * @param adjustment The GtkAdjustment to use, or NULL to create a 
+	 * @param adjustment The GtkAdjustment to use, or null to create a 
 	 * new adjustment.
 	 */
 	public HScrollBar(Adjustment adjustment){
-		super(gtk_hscrollbar_new( adjustment.getHandle() ));
+		super(gtk_hscrollbar_new(adjustment == null ? null : adjustment.getHandle()));
 	}
 	
 	/**
@@ -37,6 +38,21 @@ public class HScrollBar extends ScrollBar
 	public HScrollBar(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Construct a HScrollBar from a handle to a native resource.
+     */
+    public static HScrollBar getHScrollBar(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        HScrollBar obj = (HScrollBar) GObject.getGObjectFromHandle(handle);
+        
+        if (obj == null)
+            obj = new HScrollBar(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Retrieve the runtime type used by the GLib library.
