@@ -10,6 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Type;
+import org.gnu.glib.Handle;
 
 /**
  * The HSeparator widget is a horizontal separator, used to group the widgets 
@@ -32,9 +33,23 @@ public class HSeparator extends Separator
 	/**
 	 * Construct a HSeparator from a handle to a native resource.
 	 */
-	public HSeparator(int handle) {
+	public HSeparator(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static HSeparator getHSeparator(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        HSeparator obj = (HSeparator) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new HSeparator(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Retrieve the runtime type used by the GLib library.
@@ -43,13 +58,9 @@ public class HSeparator extends Separator
 		return new Type(gtk_hseparator_get_type());
 	}
 
-    /****************************************
-     * BEGINNING OF GENERATED CODE
-     ****************************************/
+
     native static final protected int gtk_hseparator_get_type ();
-    native static final protected int gtk_hseparator_new ();
-    /****************************************
-     * END OF GENERATED CODE
-     ****************************************/
+    native static final protected Handle gtk_hseparator_new ();
+
 }
 
