@@ -9,9 +9,8 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.glib.Handle;
 
 /**
  * This widget is similar to DrawingArea in that it's a "blank slate" and 
@@ -40,6 +39,20 @@ public class Layout extends Container
 	public Layout(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Construct a new Layout from a handle to a native resource.
+     */
+    public static Layout getLayout(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Layout obj = (Layout) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new Layout(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Add a child Widget to this Layout.
@@ -90,10 +103,7 @@ public class Layout extends Container
 	 */
 	public Adjustment getHorizontalAdjustment() {
 	    Handle han = Layout.gtk_layout_get_hadjustment(getHandle());
-		GObject obj = retrieveGObject(han);
-		if (null != obj)
-			return (Adjustment)obj;
-		return new Adjustment(han);
+		return Adjustment.getAdjustment(han);
 	}
 	
 	/**
@@ -108,10 +118,7 @@ public class Layout extends Container
 	 */
 	public Adjustment getVerticalAdjustment() {
 	    Handle han = Layout.gtk_layout_get_vadjustment(getHandle());
-		GObject obj = retrieveGObject(han);
-		if (null != obj)
-			return (Adjustment)obj;
-		return new Adjustment(han);
+		return Adjustment.getAdjustment(han);
 	}
 	
 	/**
@@ -140,5 +147,9 @@ public class Layout extends Container
     native static final protected void gtk_layout_set_hadjustment (Handle layout, Handle adjustment);
     native static final protected void gtk_layout_set_vadjustment (Handle layout, Handle adjustment);
 
+    /* Deprecated functions.
+    native static final private void gtk_layout_freeze(Handle layout);
+    native static final private void gtk_layout_thaw(Handle layout);
+     */
 }
 
