@@ -9,10 +9,14 @@
 
 package org.gnu.gtk;
 
+import java.util.ArrayList;
 import java.util.HashMap;
-import org.gnu.glib.Handle;
+import java.util.List;
+
+import org.gnu.gdk.Pixbuf;
 import org.gnu.glib.Type;
 import org.gnu.glib.Value;
+import org.gnu.glib.Handle;
 
 /**
  * The ListStore is the gtk Model used for constructing Lists and tables to be
@@ -35,6 +39,7 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 {
 	// Hash to store the TreeIterComparison objects used to sort a column.
 	protected HashMap sortMethods = null;
+    private List columnTypes = new ArrayList();
 
 	/**
 	 * Constructs a new ListStore object for storing data in a list style
@@ -46,10 +51,14 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * later.
 	 */
 	public ListStore(DataColumn[] dataColumns){
-		super(init(dataColumns));
+		super(initListStore(dataColumns));
+        
+		for (int i=0; i<dataColumns.length; i++){
+            columnTypes.add(dataColumns[i]);
+        }
 	}
 	
-	private static Handle init(DataColumn[] dataColumns) { 
+	private static Handle initListStore(DataColumn[] dataColumns) { 
 	    int[] intTypes = new int[ dataColumns.length ];
 		for (int i=0; i<intTypes.length; i++){
 			intTypes[i] = dataColumns[i].getType().getTypeHandle();
@@ -57,16 +66,201 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 		}
 		return gtk_list_store_newv(dataColumns.length, intTypes);
 	}
+    
+    /**
+     * Adds a row to the list. This method can be used to easily add data
+     * to the list. Each item in the DataRow must match the {@link DataColumn}
+     * type specified in the constructor of ListStore. For example, if
+     * it was passed <code>{new DataColumnBoolean(), new DataColumnString()}</code> to the
+     * constructor, then the DataRow must contain a Boolean and a String object,
+     * respectively.
+     * @param aRow a row to be added to the list
+     */
+	public void addRow(DataRow aRow) {
+        
+        TreeIter rowIter = this.appendRow();
+        int size = columnTypes.size();
+        
+        for (int i=0; i < size; i++) {
+            
+            DataColumn dataCol = getDataColumn(i);
+            Object rowItem = aRow.get(i);
+            
+//          TODO: should use gtk_list_store_set_valist instead
+            
+            trySetValue(rowIter, dataCol, rowItem);
+        }
+    }
 
+	/**
+     * Adds a row to the list, at the specified position. 
+     * This method can be used to easily add data
+     * to the list. Each item in the DataRow must match the {@link DataColumn}
+     * type specified in the constructor of ListStore. For example, if
+     * it was passed <code>{new DataColumnBoolean(), new DataColumnString()}</code> to the
+     * constructor, then the DataRow must contain a Boolean and a String object,
+     * respectively.
+     * @param aRow a row to be added to the list
+     */
+    public void addRowAt(DataRow aRow, int aPosition) {
+        
+        TreeIter rowIter = this.insertRow(aPosition);
+        int size = columnTypes.size();
+        
+        for (int i=0; i < size; i++) {
+            
+            DataColumn dataCol = getDataColumn(i);
+            Object rowItem = aRow.get(i);
+            
+//          TODO: should use gtk_list_store_set_valist instead
+            
+            trySetValue(rowIter, dataCol, rowItem);
+        }
+    }
+    
+    /**
+     * @param rowIter
+     * @param dataCol
+     * @param rowItem
+     */
+    private void trySetValue(TreeIter rowIter, DataColumn dataCol, Object rowItem) {
+        
+        if (dataCol instanceof DataColumnBoolean) {
+            setValue(rowIter, (DataColumnBoolean) dataCol, ((Boolean)rowItem).booleanValue() );
+            
+        } else if (dataCol instanceof DataColumnDouble) {
+            setValue(rowIter, (DataColumnDouble) dataCol, ((Double)rowItem).doubleValue() );
+            
+        } else if (dataCol instanceof DataColumnIconSize) {
+            setValue(rowIter, (DataColumnIconSize) dataCol, ((IconSize)rowItem) );
+            
+        } else if (dataCol instanceof DataColumnInt) {
+            setValue(rowIter, (DataColumnInt) dataCol, ((Integer)rowItem).intValue() );
+            
+        } else if (dataCol instanceof DataColumnObject) {
+            setValue(rowIter, (DataColumnObject) dataCol, rowItem );
+            
+        } else if (dataCol instanceof DataColumnPixbuf) {
+            setValue(rowIter, (DataColumnPixbuf) dataCol, ((Pixbuf)rowItem) );
+            
+        } else if (dataCol instanceof DataColumnStockItem) {
+            setValue(rowIter, (DataColumnStockItem) dataCol, ((GtkStockItem)rowItem) );
+            
+        } else if (dataCol instanceof DataColumnString) {
+            setValue(rowIter, (DataColumnString) dataCol, ((String)rowItem) );
+        }
+    }
+
+    /**
+     * Gets all the values in the row indicated by <code>aIter</code>. There are
+     * several ways you can get a <code>TreeIter</code> to pass; for example:
+     * using methods like <code>getIter</code> and <code>getFirstIter()</code>,
+     * or from a <code>TreeSelection</code>. 
+     * @param aIter the iter pointing to the row
+     * @return a <code>DataRow</code> filled with the values of the row.
+     * @see TreeModel#getIter(String)
+     * @see TreeModel#getIter(TreePath)
+     * @see TreeModel#getFirstIter()
+     * @see TreeSelection
+     */
+    public DataRow getRowAt(TreeIter aIter) {
+        
+        DataRow row = new DataRow();
+        int size = columnTypes.size();
+        
+        for (int i=0; i<size; i++) {
+            
+            //TODO: should use gtk_tree_model_get_valist ()
+            //instead of calling getValue for each column
+            
+            DataColumn dataCol = getDataColumn(i);
+            
+            if (dataCol instanceof DataColumnBoolean) {
+                row.add(Boolean.valueOf(getValue(aIter, (DataColumnBoolean) dataCol)));
+                
+            } else if (dataCol instanceof DataColumnDouble) {
+                row.add(new Double(getValue(aIter, (DataColumnDouble) dataCol)));
+                
+            } else if (dataCol instanceof DataColumnIconSize) {
+                row.add(new Integer(getValue(aIter, (DataColumnIconSize) dataCol)));
+                
+            } else if (dataCol instanceof DataColumnInt) {
+                row.add(new Integer(getValue(aIter, (DataColumnInt) dataCol)));
+                
+            } else if (dataCol instanceof DataColumnObject) {
+                row.add(getValue(aIter, (DataColumnObject) dataCol));
+                
+            } else if (dataCol instanceof DataColumnPixbuf) {
+                row.add(getValue(aIter, (DataColumnPixbuf) dataCol));
+                
+            } else if (dataCol instanceof DataColumnStockItem) {
+                row.add(getValue(aIter, (DataColumnStockItem) dataCol));
+                
+            } else if (dataCol instanceof DataColumnString) {
+                row.add(getValue(aIter, (DataColumnString) dataCol));
+            }
+        }
+        
+        return row;
+    }
+
+    /**
+     * Sets all columns pointed by <code>aIter</code> to the values stored
+     * in <code>aRow</code>.
+     * @param aRow a row with items
+     * @param aIter a <code>TreeIter</code> pointing to a row in the tree.
+     * @see #getRowAt(TreeIter)
+     */
+    public void setRowAt(DataRow aRow, TreeIter aIter) {
+        
+        //TODO: should use gtk_list_store_set_valist instead
+        
+        int size = columnTypes.size();
+        
+        for (int i=0; i < size; i++) {
+            trySetValue(aIter, getDataColumn(i), aRow.get(i));
+        }
+    }
+    
+    /**
+     * This function is meant primarily for GObjects that inherit from 
+     * ListStore, and should only be used when constructing a new ListStore. 
+     * It will not function after a row has been added, or a method on the 
+     * TreeModel interface is called.
+     * @param dataColumns the data columns to be set
+     */
 	public void setColumnTypes(DataColumn[] dataColumns) {
 	    int[] intTypes = new int[ dataColumns.length ];
 		for (int i=0; i<intTypes.length; i++){
 			intTypes[i] = dataColumns[i].getType().getTypeHandle();
 			dataColumns[i].setColumn(i);
+            columnTypes.add(dataColumns[i]);
 		}
-		gtk_list_store_set_column_types(getHandle(), dataColumns.length, intTypes);
+		
+        gtk_list_store_set_column_types(getHandle(), dataColumns.length, intTypes);
 	}
 
+    /**
+     * Returns the type of the column <code>aCol</code> as a 
+     * <code>DataColumn</code>. This is similar to calling 
+     * <code>getType(int)</code>; the difference is that this method returns
+     * the type as a DataColumn, which is a more high-level structure.
+     * @param aCol the column index
+     * @return the corresponding DataColumn
+     */
+    public DataColumn getDataColumn(int aCol) {
+        return (DataColumn) columnTypes.get(aCol);
+    }
+    
+    /**
+     * Returns a <code>List</code> with the types for every column as
+     * a <code>DataColumn</code> object.
+     * @return the list of <code>DataColumn</code>'s
+     */
+    public List getAllDataColumns() {
+        return columnTypes;
+    }
+
 	/**
 	 * Sets a value in the data store. To display the data in the widget, you
 	 * need to associate the datablock with the renderer, using methods of the
@@ -92,11 +286,11 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * {@link TreeView}; construct and add a {@link CellRenderer} to that; and
 	 * finally associate the properties of the CellRenderer with the dataBlocks,
 	 * using the {@link TreeViewColumn#addAttributeMapping(CellRenderer, CellRendererAttribute,
-	 * int)} method.  @param dataBlock The datablock in which the data should be
+	 * DataColumn)} method.  @param dataBlock The datablock in which the data should be
 	 * stored.
 	 * @param dataBlock The data block in which to store the data
 	 * @param iter Valid iterator for the data row in which the value is to be
-	 * set. These can be gotten using methods such as {@link #appendRow(TreeIter)}.
+	 * set. These can be gotten using methods such as {@link #appendRow()}.
 	 * @param value The value to be set. 
 	 */
 	public void setValue(TreeIter iter, DataColumnStockItem dataBlock, GtkStockItem value) {
@@ -113,11 +307,11 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * {@link TreeView}; construct and add a {@link CellRenderer} to that; and
 	 * finally associate the properties of the CellRenderer with the dataBlocks,
 	 * using the {@link TreeViewColumn#addAttributeMapping(CellRenderer, CellRendererAttribute,
-	 * int)} method.  @param dataBlock The datablock in which the data should be
+	 * DataColumn)} method.  @param dataBlock The datablock in which the data should be
 	 * stored.
 	 * @param dataBlock The data block in which to store the data
 	 * @param iter Valid iterator for the data row in which the value is to be
-	 * set. These can be gotten using methods such as {@link #appendRow(TreeIter)}.
+	 * set. These can be gotten using methods such as {@link #appendRow()}.
 	 * @param value The value to be set.
 	 */
 	public void setValue(TreeIter iter, DataColumnIconSize dataBlock, IconSize value) {
@@ -182,9 +376,8 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * the type set for that dataBlock in the constructor.  
 	 * @param dataBlock The datablock in which the data should be
 	 * stored.
-	 * @param dataBlock The data block in which to store the data
 	 * @param iter Valid iterator for the data row in which the value is to be
-	 * set. These can be gotten using methods such as {@link #appendRow(TreeIter)}.
+	 * set. These can be gotten using methods such as {@link #appendRow()}.
 	 * @param value The value to be set.
 	 */
 	public void setValue(TreeIter iter, DataColumnObject dataBlock, Object value) {
@@ -199,10 +392,8 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * make the data visible in any of the widgets which use the class 
 	 * @param dataBlock The datablock in which the data should be
 	 * stored.
-	 * @param dataBlock The data block in which to store the data, starting at
-	 * 0.
 	 * @param iter Valid iterator for the data row in which the value is to be
-	 * set. These can be gotten using methods such as {@link #appendRow(TreeIter)}.
+	 * set. These can be gotten using methods such as {@link #appendRow()}.
 	 * @param value The value to be set. This <em>must</em> match the type for
 	 * that dataBlock, as set in the constructor.
 	 */
@@ -217,10 +408,13 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * Removes the given row from the list store. After being removed, 
 	 * <code>iter</code> is set to be the next valid row, or invalidated if it 
 	 * pointed to the last row in this store.
-	 * @paramTreeIter iterator for the row to be removed.
+	 * @param iter iterator for the row to be removed.
 	 * @return TRUE if iter is valid, FALSE if not.
 	 */
 	public boolean removeRow(TreeIter iter){
+        
+        //TODO: gotta check if iter is really being set to the next row
+        
 		return gtk_list_store_remove(getHandle(), iter.getHandle());
 	}
 
@@ -230,8 +424,9 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * @param position The position to place the new row, starting at 0.
 	 * @return Iterator for the new row
 	 */
-	public TreeIter insertRow(int position){
-		return new TreeIter( gtk_list_store_insert(getHandle(), position), (TreeModel) this );
+	public TreeIter insertRow(int position) {
+        return TreeIter.getTreeIter(
+                gtk_list_store_insert(getHandle(), position), this);
 	}
 	
 	/**
@@ -241,9 +436,10 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * @param sibling 
 	 * @return Iterator for the new row
 	 */
-	public TreeIter insertRowBefore(TreeIter sibling){
-		return new TreeIter( gtk_list_store_insert_before(getHandle(), sibling.getHandle()), (TreeModel) this  );
-	}
+	public TreeIter insertRowBefore(TreeIter sibling) {
+        return TreeIter.getTreeIter(gtk_list_store_insert_before(getHandle(),
+                sibling.getHandle()), this);
+    }
 
 	/**
 	 * Inserts a new row after sibling. If sibling is NULL, then the row will be 
@@ -252,17 +448,19 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * @param sibling
 	 * @return Iterator for the new row.
 	 */
-	public TreeIter insertRowAfter(TreeIter sibling){
-		return new TreeIter( gtk_list_store_insert_after(getHandle(), sibling.getHandle()), (TreeModel) this  );
-	}
+	public TreeIter insertRowAfter(TreeIter sibling) {
+        return TreeIter.getTreeIter(gtk_list_store_insert_after(getHandle(),
+                sibling.getHandle()), this);
+    }
 
 	/**
-	 * Prepends a new row to list_store. iter will be changed to point to this 
-	 * new row. The row will be empty after this function is called. 
+	 * Prepends a new row to list_store. The <tt>iter</tt> will be
+	 * changed to point to this new row. The row will be empty
+	 * after this function is called.
 	 * @return Iterator for the new row
 	 */
 	public TreeIter prependRow(){
-		return new TreeIter( gtk_list_store_prepend(getHandle()), (TreeModel) this  );
+		return TreeIter.getTreeIter(gtk_list_store_prepend(getHandle()), this);
 	}
 
 	/**
@@ -270,7 +468,7 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * @return Iterator for the new row
 	 */
 	public TreeIter appendRow(){
-		return new TreeIter( gtk_list_store_append(getHandle()), (TreeModel) this  );
+		return TreeIter.getTreeIter(gtk_list_store_append(getHandle()), this);
 	}
 
 	/**
@@ -333,15 +531,16 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	 * Call-back method invoked by the JNI code when sorting is required.  This
 	 * is for internal use only.
 	 */
-	public int handleCompareFunc( Handle model, Handle aIter, 
-				      Handle bIter, int col ) {
-		TreeIterComparison method = TreeSortableHelper.getMethod( this, col );
-		if ( method != null ) {
-			TreeModel mdl = (TreeModel)getGObjectFromHandle( model );
-			TreeIter a = new TreeIter( aIter, mdl );
-			TreeIter b = new TreeIter( bIter, mdl );
-			return method.compareTreeIters( mdl, a, b );
-		} else {
+	public int handleCompareFunc(Handle model, Handle aIter, 
+				      Handle bIter, int col) {
+		TreeIterComparison method = TreeSortableHelper.getMethod(this, col);
+		if (method != null) {
+			TreeModel mdl = (TreeModel)getGObjectFromHandle(model);
+			TreeIter a = TreeIter.getTreeIter(aIter, mdl);
+			TreeIter b = TreeIter.getTreeIter(bIter, mdl);
+			return method.compareTreeIters(mdl, a, b);
+		}
+        else {
 			return 0;
 		}
 	}
@@ -376,9 +575,10 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	}
 
 	/**
-	 * Moves iter in this store to the position after position. iter and
-	 * position should be in the same level. Note that this function only works
-	 * with unsorted stores.
+	 * Moves iter in this store to the position after
+	 * position. The <tt>iter</tt> and <tt>position</tt> should be in the
+	 * same level. Note that this function only works with
+	 * unsorted stores.
 	 *
 	 * @since 2.2
 	 */
@@ -396,9 +596,10 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
 	}
 
 	/**
-	 * Moves iter in store to the position before position. iter and
-	 * position should be in the same level. Note that this function only works
-	 * with unsorted stores.
+	 * Moves iter in store to the position before position. The
+	 * <tt>iter</tt> and <tt>position</tt> should be in the same
+	 * level. Note that this function only works with unsorted
+	 * stores.
 	 *
 	 * @since 2.2
 	 */
@@ -432,6 +633,7 @@ public class ListStore extends TreeModel implements TreeSortable, TreeDragSource
     native static final protected void gtk_list_store_move_before(Handle listStore, Handle iter, Handle position);
 	native protected void tree_sortable_set_sort_column_id( Handle sortable, int col, int order );
 
+
 //    native static final protected int getRoot (int cptr);
 //    native final protected void setRoot (int cptr, int root);
 //    native static final protected int getTail (int cptr);
