@@ -9,8 +9,8 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.glib.Handle;
 
 /**
  * This is a base class providing allignment and padding to a number of
@@ -21,6 +21,20 @@ public class Misc extends Widget {
 	protected Misc(Handle handle) {
 		super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    protected static Misc getMisc(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Misc obj = (Misc) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new Misc(handle);
+        
+        return obj;
+    }
 
 	/**
 	 * Sets the alignment of a widget.
