@@ -10,7 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Type;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * A GtkPlug window is a top-level window that can be made available to
@@ -24,6 +24,20 @@ public class Plug extends Window
 	public Plug(Handle handle) {
 		super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static Plug getPlug(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Plug obj = (Plug) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new Plug(handle);
+        
+        return obj;
+    }
 
 	/**
 	 * Retrieve the runtime type used by the GLib library.
