@@ -10,7 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.MemStruct;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  */
