@@ -9,8 +9,9 @@
 
 package org.gnu.gtk;
 
+import org.gnu.glib.GObject;
 import org.gnu.glib.Type;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * A single RadioButton performs the same basic function as a {@link CheckButton},
@@ -124,6 +125,23 @@ public class RadioButton extends CheckButton {
 	public RadioButton(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static RadioButton getRadioButton(Handle handle) {
+        if (handle == null) {
+            return null;
+        }
+        
+        RadioButton obj = (RadioButton) GObject.getGObjectFromHandle(handle);
+        
+        if (obj == null) {
+            obj = new RadioButton(handle);
+        }
+        
+        return obj;
+    }
 
 	/**
 	 * Retrieve the runtime type used by the GLib library.
