@@ -1,7 +1,7 @@
 /*
  * Java-Gnome Bindings Library
  *
- * Copyright 1998-2004 the Java-Gnome Team, all rights reserved.
+ * Copyright 1998-2005 the Java-Gnome Team, all rights reserved.
  *
  * The Java-Gnome bindings library is free software distributed under
  * the terms of the GNU Library General Public License version 2.
@@ -10,7 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Type;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * A RadioMenuItem is a CheckMenuItem that belongs to a group.
@@ -98,9 +98,10 @@ public class RadioMenuItem extends CheckMenuItem {
 	}
 
 	/**
-	 * Construct a RadioMenuItem using a handle to a native resource.
+	 * Internal method. Construct a RadioMenuItem using a handle to a native resource
+	 * (this is public primarily so that LibGlade can construct them).
 	 */
-	RadioMenuItem(Handle handle) {
+	public RadioMenuItem(Handle handle) {
 	    super(handle);
 	}
     
