@@ -10,7 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Boxed;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 public class Requisition extends Boxed 
 {
@@ -27,6 +27,20 @@ public class Requisition extends Boxed
 	public Requisition(Handle handle) {
 		super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static Requisition getRequisition(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Requisition obj = (Requisition) Boxed.getBoxedFromHandle(handle);
+        if (obj == null)
+            obj = new Requisition(handle);
+        
+        return obj;
+    }
 	
 	public int getWidth() {
 		return getWidth(getHandle());
