@@ -11,11 +11,10 @@ package org.gnu.gtk;
 
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.ScaleEvent;
 import org.gnu.gtk.event.ScaleListener;
+import org.gnu.glib.Handle;
 
 /**
  * A Scale is a slider control used to select a numeric value. To use it, 
@@ -80,9 +79,9 @@ public abstract class Scale extends Range {
 	 * Sets a listener to be used when a format-value request is called. 
 	 */
 	public void setFormatListener(ScaleListener listener) {
-		if (null != formatListener)
-			removeFormatListener();
-		evtMap.initialize(this, ScaleEvent.Type.FORMAT_VALUE);
+		if (null == formatListener)
+			evtMap.initialize(this, ScaleEvent.Type.FORMAT_VALUE);
+		//	removeFormatListener();
 		formatListener = listener;
 	}
 	/**
@@ -130,13 +129,9 @@ public abstract class Scale extends Range {
      */
     public Layout getLayout() {
         Handle layout = gtk_scale_get_layout( getHandle() );
-        GObject obj = getGObjectFromHandle( layout );
-        if ( obj != null ) {
-            return (Layout)obj;
-        } else {
-            return new Layout( layout );
-        }
+        return Layout.getLayout(layout);
     }
+    
     /**
      * Obtains the X coordinate where the scale will draw the
      * {@link org.gnu.pango.Layout} representing the text in the scale.
