@@ -22,6 +22,20 @@ public class SeparatorToolItem extends ToolItem {
 	public SeparatorToolItem(Handle hndl) {
 		super(hndl);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static SeparatorToolItem getSeparatorToolItem(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        SeparatorToolItem obj = (SeparatorToolItem) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new SeparatorToolItem(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Sets whether the SeparatorToolItem is drawn as a line or just
