@@ -9,8 +9,8 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.glib.Handle;
 
 /**
  *
@@ -31,6 +31,20 @@ public class TearoffMenuItem extends MenuItem
 	    super(handle);
 	}
 
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static TearoffMenuItem getTearOffMenuItem(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        TearoffMenuItem obj = (TearoffMenuItem) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new TearoffMenuItem(handle);
+        
+        return obj;
+    }
+    
 	public boolean getTornOff() {
 		return getTornOff(getHandle());
 	}
