@@ -8,9 +8,10 @@
  */
 
 package org.gnu.gtk;
+
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.glib.Handle;
 /**
  * See {@link TextView} description for an overview of the related objects. 
  */
@@ -19,21 +20,35 @@ public class TextTagTable extends GObject
 	 protected TextTagTable(Handle handle){
 		 super(handle);
 	 }
+     
+     /**
+      * Internal static factory method to be used only by Java-Gnome.
+      */        
+     protected static TextTagTable getTextTagTable(Handle handle) {
+        if (handle == null)
+            return null;
 
-	 /**
-	  * Constructs a new table, containing no tags
-	  */
+        TextTagTable obj = (TextTagTable) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new TextTagTable(handle);
+
+        return obj;
+    }     
+     
+     /**
+      * Constructs a new table, containing no tags
+      */
 	 public TextTagTable(){
 		 super(gtk_text_tag_table_new ());
 	}
 
-	/**
-	 * Add a tag to the table. The tag is assigned the highest priority in the
-	 * table.
-	 *
-	 * <p>tag must not be in a tag table already, and may not have the same name
-	 * as an already-added tag.
-	 */
+	 /**
+      * Add a tag to the table. The tag is assigned the highest priority in the
+      * table.
+      *
+      * <p>tag must not be in a tag table already, and may not have the same name
+      * as an already-added tag.
+      */
 	 public void add(TextTag tag){
 		 gtk_text_tag_table_add(getHandle(), tag.getHandle());
 	 }
@@ -52,12 +67,7 @@ public class TextTagTable extends GObject
 	  */
 	 public TextTag lookup(String name){
 	     Handle tagHandle = gtk_text_tag_table_lookup(getHandle(), name);
-	     if (null == tagHandle)
-	     	return null;
-	     GObject obj = getGObjectFromHandle(tagHandle);
-	     if (null != obj)
-	     	return (TextTag)obj;
-	     return new TextTag(tagHandle);
+	     return TextTag.getTextTag(tagHandle);
 	 }
 	
 	 /**
