@@ -16,7 +16,7 @@ import org.gnu.glib.EventType;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.ToolBarEvent;
 import org.gnu.gtk.event.ToolBarListener;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 
 
@@ -37,6 +37,20 @@ public class ToolBar extends Container {
 	public ToolBar(Handle hndl) {
 		super(hndl);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static ToolBar getToolBar(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        ToolBar obj = (ToolBar) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new ToolBar(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Add a widget to the end of the toolbar.
