@@ -10,13 +10,25 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.MemStruct;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
  
 public class ToolTipsData extends MemStruct
 {
 	ToolTipsData(Handle handle) {
 		super(handle);
 	}
+    
+    static ToolTipsData getToolTipsData(Handle handle) {
+        if (handle == null) {
+            return null;
+        }
+        ToolTipsData tipsData = (ToolTipsData)
+                MemStruct.getMemStructFromHandle(handle);
+        if (tipsData == null) {
+            tipsData = new ToolTipsData(handle);
+        }
+        return tipsData;
+    }
 	
 	public ToolTips getToolTip() {
         Handle handle = getToolTip(getHandle());
@@ -36,5 +48,6 @@ public class ToolTipsData extends MemStruct
     native static final protected Handle getWidget (Handle cptr);
     native static final protected String getTipText (Handle cptr);
     native static final protected Handle gtk_tooltips_data_get (Handle widget);
+    
 }
 
