@@ -8,9 +8,11 @@
  */
 
 package org.gnu.gtk;
+
 import org.gnu.glib.Boxed;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.glib.Handle;
+
 /**
  * A tree iterator refers to a particular row of a {@link TreeView} object.
  * They are used internally by many methods, but don't have any useful methods
@@ -23,10 +25,20 @@ public class TreeIter extends Boxed
 	 * construct iter from handle to native resources. 
 	 */
 	public TreeIter(Handle handle, TreeModel model){
-		this.handle = handle;
+		super(handle);
 		this.model = model;
 	}
-	// FIXME: when do we have to free this?
+    
+    protected static TreeIter getTreeIter(Handle handle, TreeModel model) {
+        if (handle == null) {
+            return null;
+        }
+        TreeIter treeIter = (TreeIter) Boxed.getBoxedFromHandle(handle);
+        if (treeIter == null) {
+            treeIter = new TreeIter(handle, model);
+        }
+        return treeIter;
+    }
 	
 	/**
 	 * Returns the TreeModel which this Iter is associated with
@@ -39,7 +51,8 @@ public class TreeIter extends Boxed
 	 * Returns a newly-created {@link TreePath} referenced by this iter.
 	 */
 	public TreePath getPath() {
-		return new TreePath(gtk_tree_model_get_path(model.getHandle(), handle));
+        Handle handle = gtk_tree_model_get_path(model.getHandle(), getHandle());
+        return TreePath.getTreePath(handle);
 	}
 
 	/**
@@ -49,7 +62,7 @@ public class TreeIter extends Boxed
 	 * all values in the model.
 	 */
 	public TreeIter getNextIter() {
-	    Handle hndl = gtk_tree_model_iter_next(model.getHandle(), handle);
+	    Handle hndl = gtk_tree_model_iter_next(model.getHandle(), getHandle());
 		return iterFromHandle(hndl);
 	}
 
@@ -57,7 +70,7 @@ public class TreeIter extends Boxed
 	 * Returns TRUE if iter has children, FALSE otherwise.
 	 */
 	public boolean getHasChild() {
-		return gtk_tree_model_iter_has_child(model.getHandle(), handle);
+		return gtk_tree_model_iter_has_child(model.getHandle(), getHandle());
 	}
 
 	/**
@@ -65,7 +78,7 @@ public class TreeIter extends Boxed
 	 * <code>null</code> if the iter has no children.
 	 */
 	public TreeIter getFirstChild() {
-	    Handle hndl = gtk_tree_model_iter_children(model.getHandle(), handle);
+	    Handle hndl = gtk_tree_model_iter_children(model.getHandle(), getHandle());
 		return iterFromHandle(hndl);
 	}
 	
@@ -74,7 +87,7 @@ public class TreeIter extends Boxed
 	 * identified by index.
 	 */
 	public TreeIter getChild( int index) {
-	    Handle hndl = gtk_tree_model_iter_nth_child(model.getHandle(), handle, index);
+	    Handle hndl = gtk_tree_model_iter_nth_child(model.getHandle(), getHandle(), index);
 		return iterFromHandle(hndl);
 	}
 	
@@ -82,7 +95,7 @@ public class TreeIter extends Boxed
 	 * Return the parent iterator of the given child. 
 	 */
 	public TreeIter getParent() {
-	    Handle hndl = gtk_tree_model_get_path(model.getHandle(), handle);
+	    Handle hndl = gtk_tree_model_get_path(model.getHandle(), getHandle());
 		return iterFromHandle(hndl);
 	}
 
@@ -90,7 +103,7 @@ public class TreeIter extends Boxed
 	 * Returns the number of children that <code>iter</code> has
 	 */
 	public int getChildCount() {
-		return gtk_tree_model_iter_n_children(model.getHandle(), handle);
+		return gtk_tree_model_iter_n_children(model.getHandle(), getHandle());
 	}
 
 	/**
@@ -99,19 +112,13 @@ public class TreeIter extends Boxed
 	 * return value for this string.
 	 */
 	public String toString(){
-		return gtk_tree_model_get_string_from_iter( model.getHandle(), handle );
+		return gtk_tree_model_get_string_from_iter( model.getHandle(), getHandle() );
 	}
 
-
 	private TreeIter iterFromHandle(Handle hndl) {
-		if (null == hndl)
-			return null;
-		return new TreeIter(hndl, model);
+		return TreeIter.getTreeIter(hndl, model);
 	}
 
-
-
-
 	/**
 	 * Retrieve the runtime type used by the GLib library.
 	 */
@@ -120,7 +127,6 @@ public class TreeIter extends Boxed
 	}
 
     native static final protected int gtk_tree_iter_get_type ();
-    native static final protected Handle gtk_tree_iter_copy (Handle iter);
     native static final protected void gtk_tree_iter_free (Handle iter);
     native static final protected String gtk_tree_model_get_string_from_iter(Handle treeModel, Handle iter);
     native static final protected Handle gtk_tree_model_get_path(Handle treeModel, Handle iter);
