@@ -17,7 +17,7 @@ import org.gnu.glib.GObject;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.TreeSelectionEvent;
 import org.gnu.gtk.event.TreeSelectionListener;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /**
  * The TreeSelection object is a helper object to manage the selection for a 
