@@ -9,7 +9,7 @@
 
 package org.gnu.gtk;
 
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 
 /**
@@ -33,6 +33,20 @@ public class VPaned extends Paned {
 	public VPaned(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static VPaned getVPaned(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        VPaned obj = (VPaned) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new VPaned(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Retrieve the runtime type used by the GLib library.
