@@ -9,9 +9,8 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.glib.Handle;
 
 /**
  * A Viewport is a container that does not resize to fit in the
@@ -48,6 +47,20 @@ public class Viewport extends Bin
 	public Viewport(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Internal static factory method to be used by Java-Gnome only.
+     */
+    public static Viewport getViewport(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Viewport obj = (Viewport) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new Viewport(handle);
+        
+        return obj;
+    }
 
 	/**
 	 * Returns the horizontal Viewport's Adjustment.  This can be used
@@ -58,10 +71,7 @@ public class Viewport extends Bin
 	 */
 	public Adjustment getHAdjustment() {
 	    Handle hndl = gtk_viewport_get_hadjustment(getHandle());
-		GObject obj = retrieveGObject(hndl);
-		if (null != obj)
-			return (Adjustment)obj;
-		return new Adjustment(hndl);
+		return Adjustment.getAdjustment(hndl);
 	}
 
 	/**
@@ -82,10 +92,7 @@ public class Viewport extends Bin
 	 */
 	public Adjustment getVAdjustment() {
 	    Handle hndl = gtk_viewport_get_vadjustment(getHandle());
-		GObject obj = retrieveGObject(hndl);
-		if (null != obj)
-			return (Adjustment)obj;
-		return new Adjustment(hndl);
+		return Adjustment.getAdjustment(hndl);
 	}
 
 	/**
