@@ -9,10 +9,10 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Boxed;
+import org.gnu.glib.MemStruct;
 import org.gnu.glib.Handle;
 
-public class WidgetAuxInfo extends Boxed 
+public class WidgetAuxInfo extends MemStruct 
 {
     native static final protected int getX(Handle cptr);
     native static final protected int getY(Handle cptr);
