@@ -21,14 +21,10 @@ public class AttrFloat extends Attribute
 	 * Returns the value
 	 */ 
 	public double getValue(){
-		return getValue(handle);
+		return getValue(getHandle());
 	}
-    /****************************************
-     * BEGINNING OF GENERATED CODE
-     ****************************************/
+
     native static final protected double getValue (Handle obj);
-    /****************************************
-     * END OF GENERATED CODE
-     ****************************************/
+
 }
 
