@@ -9,7 +9,7 @@
 
 package org.gnu.pango;
 
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 import org.gnu.glib.Boxed;
 
 /**
