@@ -12,7 +12,7 @@ package org.gnu.pango;
 import org.gnu.glib.Boxed;
 import org.gnu.glib.GObject;
 import org.gnu.glib.MemStruct;
-import org.gnu.javagnome.Handle;
+import org.gnu.glib.Handle;
 
 /** 
  * While complete access to the layout capabilities of Pango is
@@ -60,20 +60,29 @@ public class Layout extends GObject
         super(handle);
     }
 	
+    /**
+     * Static factory method that should only be used interally by Java-Gnome.
+     */
+    public static Layout getLayoutFromHandle(Handle handle) {
+        if (handle == null) {
+            return null;
+        }
+        
+        Layout obj = (Layout) GObject.getGObjectFromHandle(handle);
+        
+        if (obj == null) {
+            obj = new Layout(handle);
+        }
+        
+        return obj;
+    }
+    
     /**
      * Retrieve the Context used for this Layout.
      */
     public Context getContext() {
         Handle hndl = pango_layout_get_context(getHandle());
-        if ( hndl != null ) {
-            GObject obj = GObject.getGObjectFromHandle( hndl );
-            if ( obj != null ) {
-                return (Context)obj;
-            } else {
-                return new Context( hndl );
-            }
-        }
-        return null;
+        return Context.getContextFromHandle(hndl);
     }
 	
     /**
@@ -143,15 +152,7 @@ public class Layout extends GObject
      */
     public AttrList getAttributes(){
         Handle hndl = pango_layout_get_attributes (getHandle());
-        if ( hndl != null ) {
-            Boxed box = Boxed.getBoxedFromHandle( hndl );
-            if ( box != null ) {
-                return (AttrList)box;
-            } else {
-                return new AttrList( hndl );
-            }
-        }
-        return null;
+        return AttrList.getAttrListFromHandle(hndl);
     }
 
     /**
