@@ -182,8 +182,8 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Drawable_gdk_1draw_1rgb_1image
     (*env)->GetByteArrayRegion(env, rgbBuf, 0, rgbBuf_len, (jbyte*)rgbBuf_g);
     rgbBuf_g[rgbBuf_len] = 0;
     
-    gdk_draw_rgb_image (drawable_g, gc_g, x, y, width, height, dith_g, rgbBuf_g, 
-            rowstride);
+    gdk_draw_rgb_image (drawable_g, gc_g, x, y, width, height, dith_g,
+    		(guchar *)rgbBuf_g, rowstride);
 }
 
 /*
@@ -210,7 +210,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Drawable_gdk_1draw_1rgb_1image_1dithalig
     rgbBuf_g[rgbBuf_len] = 0;
 
     gdk_draw_rgb_image_dithalign (drawable_g, gc_g, x, y, width, height, dith_g, 
-            rgbBuf_g, rowstride, xDith, yDith);
+            (guchar *) rgbBuf_g, rowstride, xDith, yDith);
 }
 
 /*
@@ -235,8 +235,8 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Drawable_gdk_1draw_1rgb_132_1image
     (*env)->GetByteArrayRegion(env, rgbBuf, 0, rgbBuf_len, (jbyte*)rgbBuf_g);
     rgbBuf_g[rgbBuf_len] = 0;
 
-    gdk_draw_rgb_32_image (drawable_g, gc_g, x, y, width, height, dith_g, rgbBuf_g, 
-            rowstride);
+    gdk_draw_rgb_32_image (drawable_g, gc_g, x, y, width, height, dith_g,
+    		(guchar *) rgbBuf_g, rowstride);
 }
 
 /*
@@ -262,7 +262,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Drawable_gdk_1draw_1rgb_132_1image_1dith
     rgbBuf_g[rgbBuf_len] = 0;
 
     gdk_draw_rgb_32_image_dithalign (drawable_g, gc_g, x, y, width, height, dith_g, 
-            rgbBuf_g, rowstride, xDith, yDith);
+            (guchar *) rgbBuf_g, rowstride, xDith, yDith);
 }
 
 /*
@@ -287,8 +287,8 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Drawable_gdk_1draw_1gray_1image
     (*env)->GetByteArrayRegion(env, rgbBuf, 0, rgbBuf_len, (jbyte*)rgbBuf_g);
     rgbBuf_g[rgbBuf_len] = 0;
 
-    gdk_draw_gray_image (drawable_g, gc_g, x, y, width, height, dith_g, rgbBuf_g, 
-            rowstride);
+    gdk_draw_gray_image (drawable_g, gc_g, x, y, width, height, dith_g,
+    		(guchar *) rgbBuf_g, rowstride);
 }
 
 /*
@@ -315,8 +315,8 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Drawable_gdk_1draw_1indexed_1image
     buf_g[buf_len] = 0;
     cmap_g = (GdkRgbCmap *)getPointerFromHandle(env, cmap);
 
-    gdk_draw_indexed_image (drawable_g, gc_g, x, y, width, height, dith_g, buf_g, 
-            rowstride, cmap_g);
+    gdk_draw_indexed_image (drawable_g, gc_g, x, y, width, height, dith_g,
+    		(guchar *) buf_g, rowstride, cmap_g);
 }
 
 /*
