@@ -9,19 +9,13 @@
 
 #include <jni.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
 {
 #endif
 
-
-GdkEventType GdkEvent_get_type (GdkEvent * cptr) 
-{
-    return cptr->type;
-}
-
 /*
  * Class:     org.gnu.gdk.Event
  * Method:    getType
@@ -29,7 +23,7 @@ GdkEventType GdkEvent_get_type (GdkEvent * cptr)
 JNIEXPORT jint JNICALL Java_org_gnu_gdk_Event_getType (JNIEnv *env, jclass cls, jobject obj) 
 {
     GdkEvent *obj_g = (GdkEvent *)getPointerFromHandle(env, obj);
-    return (jint) (GdkEvent_get_type (obj_g));
+    return (jint) obj_g->type;
 }
 
 /*
@@ -38,7 +32,8 @@ JNIEXPORT jint JNICALL Java_org_gnu_gdk_Event_getType (JNIEnv *env, jclass cls,
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Event_gdk_1event_1get (JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gdk_event_get ());
+    return getGBoxedHandle(env, gdk_event_get (), GDK_TYPE_EVENT, 
+                           NULL, (GBoxedFreeFunc) gdk_event_free);
 }
 
 /*
@@ -47,7 +42,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Event_gdk_1event_1get (JNIEnv *env, j
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Event_gdk_1event_1peek (JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gdk_event_peek ());
+    return getGBoxedHandle(env, gdk_event_peek (), GDK_TYPE_EVENT,
+                           NULL, (GBoxedFreeFunc) gdk_event_free);
 }
 
 /*
@@ -58,7 +54,9 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Event_gdk_1event_1get_1graphics_1expo
     jclass cls, jobject window) 
 {
     GdkWindow *window_g = (GdkWindow *)getPointerFromHandle(env, window);
-    return getHandleFromPointer(env, gdk_event_get_graphics_expose (window_g));
+    return getGBoxedHandle(env, gdk_event_get_graphics_expose (window_g),
+                           GDK_TYPE_EVENT, (GBoxedCopyFunc) gdk_event_copy, 
+                           (GBoxedFreeFunc) gdk_event_free);
 }
 
 /*
@@ -79,7 +77,9 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Event_gdk_1event_1put (JNIEnv *env, jcla
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Event_gdk_1event_1new
   (JNIEnv *env, jclass cls, jint type)
 {
-	return getHandleFromPointer(env, gdk_event_new((GdkEventType)type));
+	return getGBoxedHandle(env, gdk_event_new((GdkEventType)type),
+                               GDK_TYPE_EVENT,
+                               NULL, (GBoxedFreeFunc) gdk_event_free);
 }
 
 
@@ -91,18 +91,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Event_gdk_1event_1copy (JNIEnv *env,
     event) 
 {
     GdkEvent *event_g = (GdkEvent *)getPointerFromHandle(env, event);
-    return getHandleFromPointer(env, gdk_event_copy (event_g));
-}
-
-/*
- * Class:     org.gnu.gdk.Event
- * Method:    gdk_event_free
- */
-JNIEXPORT void JNICALL Java_org_gnu_gdk_Event_gdk_1event_1free (JNIEnv *env, jclass cls, jobject 
-    event) 
-{
-    GdkEvent *event_g = (GdkEvent *)getPointerFromHandle(env, event);
-    gdk_event_free (event_g);
+    return getGBoxedHandle(env, gdk_event_copy (event_g),
+                           GDK_TYPE_EVENT, NULL, (GBoxedFreeFunc) gdk_event_free);
 }
 
 /*
