@@ -10,34 +10,13 @@
 #include <jni.h>
 #include <sys/types.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
 {
 #endif
 
-GdkEventType GdkEventProperty_get_type (GdkEventProperty * cptr) 
-{
-    return cptr->type;
-}
-
-/*
- * Class:     org.gnu.gdk.EventProperty
- * Method:    getType
- */
-JNIEXPORT jint JNICALL Java_org_gnu_gdk_EventProperty_getType (JNIEnv *env, jclass cls, jobject 
-    obj) 
-{
-    GdkEventProperty *obj_g = (GdkEventProperty *)getPointerFromHandle(env, obj);
-    return (jint) (GdkEventProperty_get_type (obj_g));
-}
-
-GdkWindow * GdkEventProperty_get_window (GdkEventProperty * cptr) 
-{
-    return cptr->window;
-}
-
 /*
  * Class:     org.gnu.gdk.EventProperty
  * Method:    getWindow
@@ -46,12 +25,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_EventProperty_getWindow (JNIEnv *env,
     obj) 
 {
     GdkEventProperty *obj_g = (GdkEventProperty *)getPointerFromHandle(env, obj);
-    return getHandleFromPointer(env, GdkEventProperty_get_window (obj_g));
-}
-
-gboolean GdkEventProperty_get_send_event (GdkEventProperty * cptr) 
-{
-    return cptr->send_event;
+    return getGObjectHandle(env, (GObject *)obj_g->window);
 }
 
 /*
@@ -62,12 +36,7 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_EventProperty_getSendEvent (JNIEnv *
     cls, jobject obj) 
 {
     GdkEventProperty *obj_g = (GdkEventProperty *)getPointerFromHandle(env, obj);
-    return (jboolean) (GdkEventProperty_get_send_event (obj_g));
-}
-
-GdkAtom * GdkEventProperty_get_atom (GdkEventProperty * cptr) 
-{
-    return (GdkAtom *)cptr->atom;
+    return (jboolean) obj_g->send_event;
 }
 
 /*
@@ -78,12 +47,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gdk_EventProperty_getAtom (JNIEnv *env, jcla
     obj) 
 {
     GdkEventProperty *obj_g = (GdkEventProperty *)getPointerFromHandle(env, obj);
-    return (jint)GdkEventProperty_get_atom (obj_g);
-}
-
-time_t GdkEventProperty_get_time (GdkEventProperty * cptr) 
-{
-    return cptr->time;
+    return (jint) obj_g->atom;
 }
 
 /*
@@ -94,12 +58,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gdk_EventProperty_getTime (JNIEnv *env, jcla
     obj) 
 {
     GdkEventProperty *obj_g = (GdkEventProperty *)getPointerFromHandle(env, obj);
-    return GdkEventProperty_get_time (obj_g);
-}
-
-guint32 GdkEventProperty_get_state (GdkEventProperty * cptr) 
-{
-    return cptr->state;
+    return obj_g->time;
 }
 
 /*
@@ -110,7 +69,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gdk_EventProperty_getState (JNIEnv *env, jcl
     obj) 
 {
     GdkEventProperty *obj_g = (GdkEventProperty *)getPointerFromHandle(env, obj);
-    return (jint) (GdkEventProperty_get_state (obj_g));
+    return (jint) obj_g->state;
 }
 
 
