@@ -32,7 +32,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gdk_Keymap_gdk_1keymap_1get_1type (JNIEnv *e
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Keymap_gdk_1keymap_1get_1default (JNIEnv *env, jclass 
     cls) 
 {
-    return getGObjectHandle(env, gdk_keymap_get_default ());
+    return getGObjectHandle(env, G_OBJECT(gdk_keymap_get_default ()));
 }
 
 /*
