@@ -133,8 +133,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1from_1drawab
 {
     GdkDrawable *src_g = (GdkDrawable *)getPointerFromHandle(env, src);
     GdkColormap *cmap_g = (GdkColormap *)getPointerFromHandle(env, cmap);
-    return getGObjectHandle(env, gdk_pixbuf_get_from_drawable (NULL, src_g, cmap_g, 
-    		srcX, srcY, destX, destY, width, height));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_get_from_drawable (NULL,
+    		src_g, cmap_g, srcX, srcY, destX, destY, width, height));
 }
 
 /*
@@ -147,8 +147,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1from_1image
 {
     GdkImage *src_g = (GdkImage *)getPointerFromHandle(env, src);
     GdkColormap *cmap_g = (GdkColormap *)getPointerFromHandle(env, cmap);
-    return getGObjectHandle(env, gdk_pixbuf_get_from_image (NULL, src_g, cmap_g, 
-    		srcX, srcY, destX, destY, width, height));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_get_from_image (NULL,
+    		src_g, cmap_g, srcX, srcY, destX, destY, width, height));
 }
 
 /*
@@ -256,8 +256,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new (JNIEnv *env,
     gint32 bitsPerSample_g = (gint32) bitsPerSample;
     gint32 width_g = (gint32) width;
     gint32 height_g = (gint32) height;
-    return getGObjectHandle(env, gdk_pixbuf_new (colorspace_g, hasAlpha_g, bitsPerSample_g, width_g, 
-            height_g));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_new (colorspace_g,
+    		hasAlpha_g, bitsPerSample_g, width_g, height_g));
 }
 
 /*
@@ -267,7 +267,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new (JNIEnv *env,
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1copy (JNIEnv *env, jclass cls, jobject pixbuf) 
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    return getGObjectHandle(env, gdk_pixbuf_copy (pixbuf_g));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_copy (pixbuf_g));
 }
 
 /*
@@ -282,7 +282,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1subpixbuf (J
     gint32 srcY_g = (gint32) srcY;
     gint32 width_g = (gint32) width;
     gint32 height_g = (gint32) height;
-    return getGObjectHandle(env, 
+    return getGObjectHandle(env, (GObject *)
     		gdk_pixbuf_new_subpixbuf (srcPixbuf_g, srcX_g, srcY_g, width_g, height_g));
 }
 
@@ -295,10 +295,12 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1file (
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
     GError* error_g = NULL;
-    jobject result =  getGObjectHandle(env, gdk_pixbuf_new_from_file (filename_g, &error_g));
+    jobject result =  getGObjectHandle(env, (GObject *)
+    		gdk_pixbuf_new_from_file (filename_g, &error_g));
     (*env)->ReleaseStringUTFChars(env, filename, filename_g);
-    if (NULL != error_g)
-    	updateHandle(env, error, error_g);
+    if (NULL != error_g){
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
+    }
     return result;
 }
 
@@ -311,10 +313,12 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1file_1
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
     GError* error_g = NULL;
-    jobject result =  getGObjectHandle(env, gdk_pixbuf_new_from_file_at_size (filename_g, width, height, &error_g));
+    GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (filename_g, width,
+    		height, &error_g);
+    jobject result =  getGObjectHandle(env, (GObject *) pixbuf);
     (*env)->ReleaseStringUTFChars(env, filename, filename_g);
     if (NULL != error_g)
-    	updateHandle(env, error, error_g);
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
     return result;
 }
 
@@ -327,10 +331,12 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1file_1
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
     GError* error_g = NULL;
-    jobject result =  getGObjectHandle(env, gdk_pixbuf_new_from_file_at_scale (filename_g, width, height, aspect, &error_g));
+    jobject result =  getGObjectHandle(env, (GObject *)
+    		gdk_pixbuf_new_from_file_at_scale (filename_g, width, height, aspect,
+    		&error_g));
     (*env)->ReleaseStringUTFChars(env, filename, filename_g);
     if (NULL != error_g)
-    	updateHandle(env, error, error_g);
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
     return result;
 }
 
@@ -364,7 +370,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1xpm_1d
     const gchar** data_g = (const gchar**)g_malloc(data_len + 1);
     (*env)->GetByteArrayRegion(env, data, 0, data_len, (jbyte*)data_g);
     data_g[data_len] = 0;
-    return getGObjectHandle(env, gdk_pixbuf_new_from_xpm_data (data_g));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_new_from_xpm_data (data_g));
 }
 
 /*
@@ -381,10 +387,10 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1inline
     GError* error_g = NULL;
     (*env)->GetByteArrayRegion(env, data, 0, data_len, (jbyte*)data_g);
     data_g[data_len] = 0;
-    jobject handle = getGObjectHandle(env, 
+    jobject handle = getGObjectHandle(env, (GObject *)
     	gdk_pixbuf_new_from_inline (dataLength_g, data_g, copyPixels_g, &error_g));
     if (NULL != error_g)
-    	updateHandle(env, error, error_g);
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
     return handle;
 }
 
@@ -393,7 +399,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1inline
  * Method:    gdk_pixbuf_savev
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1savev
-  (JNIEnv *env, jclass cls, jobject pixbuf, jstring filename, jstring type, jobjectArray keys, jobjectArray values, jobject error)
+  (JNIEnv *env, jclass cls, jobject pixbuf, jstring filename, jstring type,
+  		jobjectArray keys, jobjectArray values, jobject error)
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
     const gchar* f = (*env)->GetStringUTFChars(env, filename, NULL);
@@ -403,7 +410,7 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1savev
     GError* error_g = NULL;
 	jboolean val = gdk_pixbuf_savev(pixbuf_g, f, t, k, v, &error_g);
     if (NULL != error_g)
-    	updateHandle(env, error, error_g);
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
 	(*env)->ReleaseStringUTFChars(env, filename, f);
 	(*env)->ReleaseStringUTFChars(env, type, t);
 	freeStringArray(env, keys, k);
@@ -415,24 +422,32 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1savev
  * Class:     org_gnu_gdk_Pixbuf
  * Method:    gdk_pixbuf_save_to_bufferv
  */
-JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1save_1to_1bufferv
-  (JNIEnv *env, jclass cls, jobject pixbuf, jstring buffer, jstring type, jobjectArray keys, jobjectArray values, jobject error)
+JNIEXPORT jbyteArray JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1save_1to_1bufferv
+  (JNIEnv *env, jclass cls, jobject pixbuf, jstring type,
+  		jobjectArray keys, jobjectArray values, jobject error)
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    gchar* b = (gchar*)(*env)->GetStringUTFChars(env, buffer, NULL);
+    gchar* b = NULL;
     const gchar* t = (*env)->GetStringUTFChars(env, type, NULL);
     gchar** k = getStringArray(env, keys);
     gchar** v = getStringArray(env, values);
     GError* error_g = NULL;
     gsize size;
+    jbyteArray ret_val;
+    ret_val = NULL;
 	jboolean val = gdk_pixbuf_save_to_bufferv(pixbuf_g, &b, &size, t, k, v, &error_g);
     if (NULL != error_g)
-    	updateHandle(env, error, error_g);
-	(*env)->ReleaseStringUTFChars(env, buffer, b);
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
+    else	{
+    	ret_val = (*env)->NewByteArray(env, (jsize) size);
+    	(*env)->SetByteArrayRegion(env, ret_val, 0, (jsize) size, (jbyte *) b); 
+    	
+    }
 	(*env)->ReleaseStringUTFChars(env, type, t);
 	freeStringArray(env, keys, k);
 	freeStringArray(env, values, v);
-	return val;
+	
+	return ret_val;
 }
 
 /*
@@ -443,7 +458,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1rotate_1simple
   (JNIEnv *env, jclass cls, jobject pixbuf, jint direction)
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    return getGObjectHandle(env, gdk_pixbuf_rotate_simple(pixbuf_g, (GdkPixbufRotation)direction));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_rotate_simple(pixbuf_g,
+    		(GdkPixbufRotation) direction));
 }
 
 /*
@@ -454,7 +470,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1flip
   (JNIEnv *env, jclass cls, jobject pixbuf, jboolean horizontal)
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    return getGObjectHandle(env, gdk_pixbuf_flip(pixbuf_g, (gboolean)horizontal));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_flip(pixbuf_g,
+    		(gboolean)horizontal));
 }  
 
 /*
@@ -482,7 +499,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1add_1alpha (JNIEn
     gint32 g_g = (gint32) g;
     gint32 b_g = (gint32) b;
     // returned pixbuf has ref count of 1.  should we use a different getGObjectHandle function here?
-    return getGObjectHandle(env,
+    return getGObjectHandle(env, (GObject *)
     		gdk_pixbuf_add_alpha (pixbuf_g, substituteColor_g, r_g, g_g, b_g));
 }
 
@@ -610,7 +627,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1scale_1simple (JN
     gint32 destWidth_g = (gint32) destWidth;
     gint32 destHeight_g = (gint32) destHeight;
     GdkInterpType interpType_g = (GdkInterpType) interpType;
-    return getGObjectHandle(env, 
+    return getGObjectHandle(env, (GObject *)
     		gdk_pixbuf_scale_simple (src_g, destWidth_g, destHeight_g, interpType_g));
 }
 
@@ -630,8 +647,9 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1composite_1color_
     gint32 checkSize_g = (gint32) checkSize;
     gint32 color1_g = (gint32) color1;
     gint32 color2_g = (gint32) color2;
-    return getGObjectHandle(env, gdk_pixbuf_composite_color_simple (src_g, destWidth_g, destHeight_g, 
-            interpType_g, overallAlpha_g, checkSize_g, color1_g, color2_g));
+    return getGObjectHandle(env, G_OBJECT(gdk_pixbuf_composite_color_simple
+    		(src_g, destWidth_g, destHeight_g, interpType_g, overallAlpha_g,
+    		checkSize_g, color1_g, color2_g)));
 }
 
 /*
