@@ -11,7 +11,8 @@
 #include <string.h>
 #include <glib.h>
 #include <glib-object.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 #ifdef __cplusplus
 extern "C" 
 {
@@ -25,13 +26,9 @@ typedef struct {
   jobject methodData;  /* user defined data to pass to callback method */
   jboolean isStatic;  /* is the callback a static method? */
   jboolean isBoolReturn; /* does the callback method return a boolean value */
+  jboolean isStringReturn; /* does the callback method return a string value */
 } CallbackData;
 
-typedef struct{
-	JNIEnv *env;
-	jobject object;
-} JGRef;
-
 typedef struct {
   GClosure closure;
   CallbackData *cbdata;
@@ -61,9 +58,10 @@ jg_signal_cb(GClosure *closure,
   gboolean b;
   const gchar* str;
   const gchar* jtype;
-  int jargsLen = n_param_values;  
+  int jargsLen = n_param_values;
+  gpointer boxed = NULL;
+  GType value_type;
   
-
   cbdata = jgc->cbdata;
   if (cbdata == NULL) {
     g_critical("Java-GNOME - unable to determine the callback method.\n");
@@ -78,7 +76,8 @@ jg_signal_cb(GClosure *closure,
 
   /* build the parameters for the callback */
   for(i = 1; i < n_param_values; i++) {
-    switch(G_TYPE_FUNDAMENTAL(G_VALUE_TYPE(&param_values[i]))) {
+  	value_type = G_VALUE_TYPE(&param_values[i]);
+    switch(G_TYPE_FUNDAMENTAL(value_type)) {
     case G_TYPE_CHAR:
       jargs[i-1].c = g_value_get_char(&param_values[i]);
       break;
@@ -118,16 +117,32 @@ jg_signal_cb(GClosure *closure,
       localObj = (*cbdata->env)->NewStringUTF(cbdata->env, str);
       jargs[i-1].l = localObj;
       break;
+    case G_TYPE_OBJECT:
+      peer = getGObjectHandle(cbdata->env, 
+                              g_value_get_object(&param_values[i]));
+      jargs[i-1].l = peer;
+      break;
     case G_TYPE_BOXED:
-      peer = getHandleFromPointer(cbdata->env, g_value_get_boxed(&param_values[i]));
+      // MEM_MGT_NOTE: we don't know what GBoxed{Copy,Free}Func to use here!
+        
+      //We do the copying manually here and the GBoxed will be freed
+      //automatically by calling g_boxed_free
+      
+      boxed = g_boxed_copy(value_type, g_value_get_boxed(&param_values[i]));
+      
+      peer = getGBoxedHandle(cbdata->env, boxed,
+                             value_type, NULL, NULL);
+      
       jargs[i-1].l = peer;
       break;
     case G_TYPE_POINTER:
+      // MEM_MGT_NOTE: we don't know what free func to use here!
       peer = getHandleFromPointer(cbdata->env, g_value_get_pointer(&param_values[i]));
       jargs[i-1].l = peer;
       break;
     default:
-      peer = getHandleFromPointer(cbdata->env, g_value_get_object(&param_values[i]));
+      peer = getGObjectHandle(cbdata->env, 
+                              g_value_get_object(&param_values[i]));
       jargs[i-1].l = peer;
       break;
     }
@@ -161,6 +176,29 @@ jg_signal_cb(GClosure *closure,
       	}
     }
   }
+  
+  else if (cbdata->isStringReturn == JNI_TRUE)	{
+  	const gchar* ret_chars;
+  	jstring ret_string;
+  	if (cbdata->isStatic == JNI_FALSE)	{
+  		ret_string = (jstring) (* cbdata->env)->CallObjectMethodA(cbdata->env,
+  								cbdata->obj,
+  								mid,
+  								jargs);
+  	}
+  	else	{
+  		ret_string = (jstring) (* cbdata->env)->CallObjectMethodA(cbdata->env,
+  								cbdata->class,
+  								mid,
+  								jargs);
+  	}
+  	if (return_value)	{
+  		ret_chars = (*cbdata->env)->GetStringUTFChars(cbdata->env, ret_string, 0);
+  		g_value_set_string(return_value, ret_chars);
+  		(*cbdata->env)->ReleaseStringUTFChars(cbdata->env, ret_string, ret_chars);
+  	}
+  }
+  	
   else {
     if (cbdata->isStatic == JNI_FALSE)
       (* cbdata->env)->CallVoidMethodA(cbdata->env, 
@@ -348,6 +386,12 @@ jg_cbdata_new(JNIEnv *env,
   else
     cbdata->isBoolReturn = JNI_FALSE;
 
+  if (G_TYPE_FUNDAMENTAL(siginfo.return_type) == G_TYPE_STRING)
+    cbdata->isStringReturn = JNI_TRUE;
+  
+  else
+    cbdata->isStringReturn = JNI_FALSE;
+    
   /* add the method data to the struct if it exists */
   if (method_data != NULL)
     cbdata->methodData = (*env)->NewGlobalRef(env, method_data);
@@ -447,48 +491,51 @@ static jint create_callback(JNIEnv *env,
                             jobject cbrecv, 
                             jobject data)
 { 
-  jint retval;
-  const char *signame;
-  const char *sigfunc;
-  CallbackData *cbdata;
-  GObject *go;
-  guint sigid;
-  GQuark detail = 0;
-  GClosure *closure;
+	jint retval;
+  	const char *signame;
+  	const char *sigfunc;
+  	CallbackData *cbdata;
+  	GObject *go;
+  	guint sigid;
+  	GQuark detail = 0;
+  	GClosure *closure;
+  	jclass Exception;
 
-  jclass Exception;
-
-//  go = (GObject*)getPointerFromHandle(env, obj);
-  go = (GObject*)getPointerFromJavaGObject(env, obj);
-  signame = (*env)->GetStringUTFChars(env, sigstring, NULL);
-  sigfunc = (*env)->GetStringUTFChars(env, func, NULL);
+	go = (GObject*)getPointerFromJavaGObject(env, obj);
+	signame = (*env)->GetStringUTFChars(env, sigstring, NULL);
+	sigfunc = (*env)->GetStringUTFChars(env, func, NULL);
   
-
-  if (!g_signal_parse_name(signame, G_OBJECT_TYPE(go),
+	gdk_threads_enter();
+	if (!g_signal_parse_name(signame, G_OBJECT_TYPE(go),
 			   &sigid, &detail, TRUE)) {
-    g_critical("Java-GNOME - unknown signal name %s, function %s, object %s.\n", signame, sigfunc, G_OBJECT_TYPE_NAME(go));
-	Exception = (* env)->FindClass(env, "java/lang/RuntimeException");
-    (* env)->ThrowNew(env, Exception, "" );
-    return 0;
-  }
-  cbdata = jg_cbdata_new(env, obj, cbrecv, data, signame, sigfunc);
+    	g_critical("Java-GNOME - unknown signal name %s, function %s, object %s.\n", signame, sigfunc, G_OBJECT_TYPE_NAME(go));
+    	gdk_threads_leave();
+    	
+		Exception = (* env)->FindClass(env, "java/lang/RuntimeException");
+    	(* env)->ThrowNew(env, Exception, "" );
+  		(*env)->ReleaseStringUTFChars(env, sigstring, signame);
+  		(*env)->ReleaseStringUTFChars(env, func, sigfunc);  
+    	return 0;
+  	}
+  	cbdata = jg_cbdata_new(env, obj, cbrecv, data, signame, sigfunc);
 
-  if (cbdata == NULL)
-    return 0;
+  	if (cbdata == NULL) {
+  		gdk_threads_leave();
+		(*env)->ReleaseStringUTFChars(env, sigstring, signame);
+		(*env)->ReleaseStringUTFChars(env, func, sigfunc);  
+    	return 0;
+  	}
 
-  closure = jg_closure_new();
-  ((JGClosure *)closure)->cbdata = cbdata;
+  	closure = jg_closure_new();
+  	((JGClosure *)closure)->cbdata = cbdata;
 
-  //retval = g_signal_connect_closure_by_id(getPointerFromHandle(env, obj),
-  retval = g_signal_connect_closure_by_id(getPointerFromJavaGObject(env, obj),
-					  sigid,
-					  detail,
-					  closure,
-					  FALSE);
+	retval = g_signal_connect_closure_by_id(go,
+					  sigid, detail, closure, FALSE);
+	gdk_threads_leave();
 
-  (*env)->ReleaseStringUTFChars(env, sigstring, signame);
-  (*env)->ReleaseStringUTFChars(env, func, sigfunc);
-  return retval;
+	(*env)->ReleaseStringUTFChars(env, sigstring, signame);
+	(*env)->ReleaseStringUTFChars(env, func, sigfunc);
+	return retval;
 }
 
 
@@ -499,46 +546,85 @@ static jint create_static_callback(JNIEnv *env,
                                    jclass cbrecv, 
                                    jobject data)
 { 
-  jint retval;
-  const char *signame;
-  const char *sigfunc;
-  CallbackData *cbdata;
-  GObject *go;
-  guint sigid;
-  GQuark detail = 0;
-  GClosure *closure;
-  jclass Exception;
+	jint retval;
+	const char *signame;
+	const char *sigfunc;
+	CallbackData *cbdata;
+	GObject *go;
+	guint sigid;
+	GQuark detail = 0;
+	GClosure *closure;
+	jclass Exception;
   
-  go = (GObject*)getPointerFromJavaGObject(env, obj);
-  signame = (*env)->GetStringUTFChars(env, sigstring, NULL);
-  sigfunc = (*env)->GetStringUTFChars(env, func, NULL);
+	go = (GObject*)getPointerFromJavaGObject(env, obj);
+	signame = (*env)->GetStringUTFChars(env, sigstring, NULL);
+	sigfunc = (*env)->GetStringUTFChars(env, func, NULL);
   
-  if (!g_signal_parse_name(signame, G_OBJECT_TYPE(go),
+	gdk_threads_enter();
+	if (!g_signal_parse_name(signame, G_OBJECT_TYPE(go),
 			   &sigid, &detail, TRUE)) {
-    g_critical("Java-GNOME - unknown signal name %s, function %s.\n", signame, sigfunc);
-	Exception = (* env)->FindClass(env, "java/lang/RuntimeException");
-    (* env)->ThrowNew(env, Exception, "" );
-    return 0;
-  }
-  cbdata = jg_static_cbdata_new(env, obj, cbrecv, data, signame, sigfunc);
+    	g_critical("Java-GNOME - unknown signal name %s, function %s.\n", signame, sigfunc);
+    	gdk_threads_leave();
+    	
+		Exception = (* env)->FindClass(env, "java/lang/RuntimeException");
+    	(* env)->ThrowNew(env, Exception, "" );
+  		(*env)->ReleaseStringUTFChars(env, sigstring, signame);
+  		(*env)->ReleaseStringUTFChars(env, func, sigfunc);  
+    	return 0;
+  	}
+	cbdata = jg_static_cbdata_new(env, obj, cbrecv, data, signame, sigfunc);
 
-  if (cbdata == NULL)
-    return 0;
-    
-  closure = jg_closure_new();
-  ((JGClosure *)closure)->cbdata = cbdata;
-  retval = g_signal_connect_closure_by_id(getPointerFromJavaGObject(env, obj),
-					  sigid,
-					  detail,
-					  closure,
-					  FALSE);
+  	if (cbdata == NULL) {
+  		gdk_threads_leave();
+		(*env)->ReleaseStringUTFChars(env, sigstring, signame);
+		(*env)->ReleaseStringUTFChars(env, func, sigfunc);  
+    	return 0;
+  	}
+  	
+  	closure = jg_closure_new();
+  	((JGClosure *)closure)->cbdata = cbdata;
+  	retval = g_signal_connect_closure_by_id(getPointerFromJavaGObject(env, obj),
+					  sigid, detail, closure, FALSE);
+  gdk_threads_leave();
   
   (*env)->ReleaseStringUTFChars(env, sigstring, signame);
-  (*env)->ReleaseStringUTFChars(env, func, sigfunc);
-    
+  (*env)->ReleaseStringUTFChars(env, func, sigfunc);  
   return retval;
 }
 
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_init
+  (JNIEnv *env, jclass cls)
+{
+	initMemoryManagement();
+    jg_atexit((GVoidFunc) processPendingGObject);
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_nativeFinalize
+  (JNIEnv *env, jclass cls, jobject handle)
+{
+    nativeFinalizeGObject(env, handle);
+}
+
+JNIEXPORT jint JNICALL Java_org_gnu_glib_GObject_nativeCollect
+  (JNIEnv *env, jclass cls)
+{
+    processPendingGObject();
+    processPendingGBoxed();
+    processPendingStruct();
+}
+
+JNIEXPORT jint JNICALL 
+Java_org_gnu_glib_GObject_g_1object_1ref(JNIEnv *env, jclass cls, jobject handle)
+{
+	g_object_ref(getPointerFromHandle(env, handle));
+}
+
+JNIEXPORT jint JNICALL 
+Java_org_gnu_glib_GObject_g_1object_1unref(JNIEnv *env, jclass cls, jobject handle)
+{
+	g_object_unref(getPointerFromHandle(env, handle));
+}
+
 
 /*
  * Class:     org_gnu_glib_GObject
@@ -623,12 +709,18 @@ Java_org_gnu_glib_GObject_addEventHandler__Ljava_lang_String_2Ljava_lang_String_
  * Signature: (I)V
  */
 JNIEXPORT void JNICALL
-Java_org_gnu_glib_GObject_removeEventHandler(JNIEnv *env, jobject obj, jint sigid)
+Java_org_gnu_glib_GObject_removeEventHandler
+    (JNIEnv *env, jobject obj, jint sigid)
 {
-  g_signal_handler_disconnect(getPointerFromHandle(env, obj), sigid);
+	GObject *obj_g;
+	
+	obj_g = (GObject*)getPointerFromJavaGObject(env, obj);
+	
+	gdk_threads_enter();
+  	g_signal_handler_disconnect(obj_g, sigid);
+  	gdk_threads_leave();
 }
 
-
 /*
  * Class:     org_gnu_glib_GObject/
  * Method:    getData
@@ -637,15 +729,22 @@ Java_org_gnu_glib_GObject_removeEventHandler(JNIEnv *env, jobject obj, jint sigi
 JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_getData
   (JNIEnv *env, jobject obj, jobject handle, jstring key)
 {
-    gchar *k = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-    GObject *handle_g = (GObject *)getPointerFromHandle(env, handle);
-    JGRef *ref = (JGRef*) g_object_get_data(handle_g, k);
+	gchar *k;
+	GObject *handle_g;
+	JGRef *ref;
+	
+    k = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
+    handle_g = (GObject *)getPointerFromHandle(env, handle);
+    
+    gdk_threads_enter();
+    ref = (JGRef*) g_object_get_data(handle_g, k);
+    gdk_threads_leave();
+    
     (*env)->ReleaseStringUTFChars(env, key, k);
-    if (ref == NULL) {
+    if (ref == NULL)
         return NULL;
-    } else {
+   	else
         return ref->object;
-    }
 }
 
 static void jg_gobject_destroy(gpointer data)
@@ -661,16 +760,23 @@ static void jg_gobject_destroy(gpointer data)
  * Signature: (ILjava/lang/String;Ljava/lang/Object;)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_setData
-(JNIEnv *env, jclass cls, jobject handle, jstring key, jobject data)
+    (JNIEnv *env, jclass cls, jobject handle, jstring key, jobject data)
 {
-    GObject *handle_g = (GObject *)getPointerFromHandle(env, handle);
-    gchar *k = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
+	GObject *handle_g;
+	const gchar *k;
+	JGRef *ref;
+	
+    handle_g = (GObject *)getPointerFromHandle(env, handle);
+    k = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 
-    JGRef *ref = g_new( JGRef, 1 );
+	gdk_threads_enter();
+    ref = g_new( JGRef, 1 );
     ref->env = env;
-    ref->object = (* env)->NewGlobalRef(env, data);
+    ref->object = (* env)->NewWeakGlobalRef(env, data);
 
     g_object_set_data_full(handle_g, k, ref, jg_gobject_destroy ) ;
+    gdk_threads_leave();
+    
     (*env)->ReleaseStringUTFChars(env, key, k);
 }
 
@@ -687,7 +793,6 @@ void logFunc(const gchar* logDomain, GLogLevelFlags logLevel, const gchar* messa
     gobject = (*env)->FindClass(env, "org/gnu/glib/GObject");
     stackHandler = (*env)->GetStaticMethodID(env, gobject, "printStackTrace", "(Ljava/lang/String;)V");
     (*env)->CallStaticVoidMethod(env, gobject, stackHandler, msg);
-	
 }
 
 /*
@@ -696,8 +801,9 @@ void logFunc(const gchar* logDomain, GLogLevelFlags logLevel, const gchar* messa
  * Signature: ()V
  */ 
 JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_registerLogHandler
-(JNIEnv *env, jclass cls)
+    (JNIEnv *env, jclass cls)
 {
+	gdk_threads_enter();
 	g_log_set_handler(NULL, G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL |
 			G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION, logFunc, (gpointer)env);
 	g_log_set_handler ("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
@@ -710,95 +816,134 @@ JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_registerLogHandler
                      | G_LOG_FLAG_RECURSION, logFunc, (gpointer) env);
 	g_log_set_handler ("Glade", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
                      | G_LOG_FLAG_RECURSION, logFunc, (gpointer) env);
-}
-
-/*
- * Class:     org_gnu_glib_GObject
- * Method:    getNullHandle
- * Signature: ()Lorg/gnu/glib/Handle;
- */
-JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_getNullHandle
-(JNIEnv *env, jclass cls)
-{
-    return getHandleFromPointer(env, NULL);
+	gdk_threads_leave();
 }
 
 JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_g_1object_1new
-(JNIEnv *env, jclass cls, jint object_type)
+    (JNIEnv *env, jclass cls, jint object_type)
 {
-    GType object_type_g = (GType)object_type;
-    return getHandleFromPointer(env, g_object_new(object_type_g, NULL, NULL));
+	GObject *obj;
+	GType object_type_g;
+	
+    object_type_g = (GType)object_type;
+    
+    gdk_threads_enter();
+    obj = g_object_new(object_type_g, NULL, NULL);
+    gdk_threads_leave();
+    
+    return getGObjectHandle(env, obj);
 }
 
 JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_g_1object_1class_1find_1property
-(JNIEnv *env, jclass cls, jobject oclass, jstring property_name)
+    (JNIEnv *env, jclass cls, jobject oclass, jstring property_name)
 {
-    GObjectClass * oclass_g = 
+    GObjectClass *oclass_g;
+    const gchar *property_name_g;
+    GParamSpec *property;
+    jobject value;
+	
+    oclass_g = 
         G_OBJECT_GET_CLASS((GObject *)getPointerFromHandle(env, oclass));
-    const gchar * property_name_g = 
+    property_name_g = 
         (const gchar *)(*env)->GetStringUTFChars(env, property_name, 0);
 
-    jobject ret_g = getHandleFromPointer(env, g_object_class_find_property(oclass_g, property_name_g));
+    gdk_threads_enter();
+    property = g_object_class_find_property(oclass_g, property_name_g);
+    gdk_threads_leave();
 
     (*env)->ReleaseStringUTFChars(env, property_name, property_name_g);
-    return ret_g;
+    // MEM_MGT_NOTE: OK. See GObject#hasGtkProperty
+    return getHandleFromPointer(env, property);	
 }
 
 JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_g_1object_1set_1property
-(JNIEnv *env, jclass cls, jobject object, jstring property_name, jobject value)
+    (JNIEnv *env, jclass cls, jobject object, jstring property_name, jobject value)
 {
-    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
-    const gchar * property_name_g = (const gchar *)(*env)->GetStringUTFChars(env, property_name, 0);
-    const GValue * value_g = (const GValue *)getPointerFromHandle(env, value);
+	GObject *object_g;
+	const gchar *property_name_g;
+	const GValue *value_g;
+	
+    object_g = (GObject *)getPointerFromHandle(env, object);
+    property_name_g = (const gchar *)(*env)->GetStringUTFChars(env, property_name, 0);
+    value_g = (const GValue *)getPointerFromHandle(env, value);
+    
+    gdk_threads_enter();
     g_object_set_property(object_g, property_name_g, value_g);
+    gdk_threads_leave();
+    
     (*env)->ReleaseStringUTFChars(env, property_name, property_name_g);
 }
 
 JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_g_1object_1get_1property
-(JNIEnv *env, jclass cls, jobject object, jstring property_name)
+    (JNIEnv *env, jclass cls, jobject object, jstring property_name)
 {
-    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
-    const gchar * property_name_g = 
+    GObject *object_g;
+    const gchar *property_name_g;
+    GParamSpec *pspec;
+    GValue *value_g;
+	
+    object_g = (GObject *)getPointerFromHandle(env, object);
+    property_name_g = 
         (const gchar *)(*env)->GetStringUTFChars(env, property_name, 0);
 
-    // Initialize the GValue.
-    GParamSpec *pspec = 
-        g_object_class_find_property(G_OBJECT_GET_CLASS(object_g), 
-                                     property_name_g);
+    gdk_threads_enter();
+    pspec = g_object_class_find_property(G_OBJECT_GET_CLASS(object_g), 
+                                         property_name_g);
     if ( !pspec ) {
+    	gdk_threads_leave();
+        (*env)->ReleaseStringUTFChars(env, property_name, property_name_g);
         return NULL;
     }
-    GValue * value_g = g_malloc( sizeof( GValue ) );
+    value_g = g_malloc( sizeof( GValue ) );
     value_g->g_type = 0;
     value_g = g_value_init(value_g, pspec->value_type);
 
     // Get the property.
     g_object_get_property(object_g, property_name_g, value_g);
+    gdk_threads_leave();
+    
     (*env)->ReleaseStringUTFChars(env, property_name, property_name_g);
-    return getHandleFromPointer(env, value_g);
+    return getStructHandle(env, value_g, NULL, g_free);
 }
 
 JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_g_1object_1freeze_1notify
 (JNIEnv *env, jclass cls, jobject object)
 {
-    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+	GObject *object_g;
+	
+    object_g = (GObject *)getPointerFromHandle(env, object);
+    
+    gdk_threads_enter();
     g_object_freeze_notify(object_g);
+    gdk_threads_leave();
 }
 
 JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_g_1object_1notify
-(JNIEnv *env, jclass cls, jobject object, jstring property_name)
+    (JNIEnv *env, jclass cls, jobject object, jstring property_name)
 {
-    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
-    const gchar * property_name_g = (const gchar *)(*env)->GetStringUTFChars(env, property_name, 0);
+	GObject *object_g;
+	const gchar *property_name_g;
+	
+    object_g = (GObject *)getPointerFromHandle(env, object);
+    property_name_g = (const gchar *)(*env)->GetStringUTFChars(env, property_name, 0);
+    
+    gdk_threads_enter();
     g_object_notify(object_g, property_name_g);
+    gdk_threads_leave();
+    
     (*env)->ReleaseStringUTFChars(env, property_name, property_name_g);
 }
 
 JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_g_1object_1thaw_1notify
-(JNIEnv *env, jclass cls, jobject object)
+    (JNIEnv *env, jclass cls, jobject object)
 {
-    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+	GObject *object_g;
+	
+    object_g = (GObject *)getPointerFromHandle(env, object);
+    
+    gdk_threads_enter();
     g_object_thaw_notify(object_g);
+    gdk_threads_leave();
 }
 
 static JGFuncCallbackRef *propertyNotifRef = NULL;
@@ -810,20 +955,26 @@ static void propertyNotificationCB(GObject *gobject, GParamSpec *arg1,
     (* ref->env)->CallVoidMethod(ref->env, 
                                  ref->obj, 
                                  ref->methodID,
-                                 getHandleFromPointer(ref->env, gobject),
+                                 getGObjectHandle(ref->env, gobject),
                                  (*ref->env)->NewStringUTF(ref->env, 
                                                            arg1->name));
 }
 
 JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_connectNotifySignal
-(JNIEnv *env, jclass cls, jobject objhandle, jobject object, jstring callback)
+    (JNIEnv *env, jclass cls, jobject objhandle, jobject object, jstring callback)
 {
+	GObject *object_g;
+	const char *funcname;
+	const char *sig = "(Lorg/gnu/glib/Handle;Ljava/lang/String;)V";
+	
+    funcname = (*env)->GetStringUTFChars(env, callback, NULL);
+    object_g = (GObject *)getPointerFromHandle(env, objhandle);
+
+	gdk_threads_enter();	
     propertyNotifRef = g_new( JGFuncCallbackRef, 1 );
     propertyNotifRef->env = env;
     propertyNotifRef->obj = (* env)->NewGlobalRef(env, object);
 
-    const char *funcname = (*env)->GetStringUTFChars(env, callback, NULL);
-    const char *sig = "(Lorg/gnu/glib/Handle;Ljava/lang/String;)V";
     // Get method id for the callback method name.
     propertyNotifRef->methodID = 
         (*env)->GetMethodID(env, 
@@ -831,49 +982,66 @@ JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_connectNotifySignal
                             funcname, sig );
     if ( propertyNotifRef->methodID == NULL ) {
         g_warning( "Can't find %s%s.\n", funcname, sig );
-        (*env)->ReleaseStringUTFChars(env, callback, funcname);
         g_free( propertyNotifRef );
         propertyNotifRef = NULL;
+        gdk_threads_leave();
+        (*env)->ReleaseStringUTFChars(env, callback, funcname);
         return;
     }
-    (*env)->ReleaseStringUTFChars(env, callback, funcname);
-
-    GObject * object_g = (GObject *)getPointerFromHandle(env, objhandle);
     g_object_connect(object_g, "signal::notify", 
                      propertyNotificationCB, propertyNotifRef, NULL);
+	gdk_threads_leave();
+
+    (*env)->ReleaseStringUTFChars(env, callback, funcname);
 }
 
 JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_disconnectNotifySignal
-(JNIEnv *env, jclass cls, jobject object)
+    (JNIEnv *env, jclass cls, jobject object)
 {
+	GObject *object_g;
+	
     if ( propertyNotifRef ) {
-        GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+        object_g = (GObject *)getPointerFromHandle(env, object);
+        
+        gdk_threads_enter();
         g_object_disconnect(object_g, "any_signal::notify",
                             propertyNotificationCB, propertyNotifRef, NULL);
         g_free( propertyNotifRef );
+        gdk_threads_leave();
     }
 }
 
-JNIEXPORT jint JNICALL Java_org_gnu_glib_GObject_getGTypeOfProperty(JNIEnv* env, jclass cls, jobject object, jstring prop) 
+JNIEXPORT jint JNICALL Java_org_gnu_glib_GObject_getGTypeOfProperty
+    (JNIEnv* env, jclass cls, jobject object, jstring prop) 
 {
-    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
-    const gchar * prop_g = 
-        (const gchar *)(*env)->GetStringUTFChars(env, prop, 0);
+	GObject *object_g;
+	const gchar *prop_g;
+	GParamSpec *pspec;
+	
+    object_g = (GObject *)getPointerFromHandle(env, object);
+    prop_g = (const gchar *)(*env)->GetStringUTFChars(env, prop, 0);
 
-    GParamSpec *pspec = 
-        g_object_class_find_property(G_OBJECT_GET_CLASS(object_g), prop_g);
-    if ( !pspec ) {
+	gdk_threads_enter();
+    pspec = g_object_class_find_property(G_OBJECT_GET_CLASS(object_g), prop_g);
+    gdk_threads_leave();
+    
+    (*env)->ReleaseStringUTFChars(env, prop, prop_g);
+    if ( !pspec )
         return -1;
-    }
-
     return pspec->value_type;
 }
 
-JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_instantiateJGObjectFromGType(JNIEnv* env, jclass cls, int type, jobject handle) 
+JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_instantiateJGObjectFromGType
+    (JNIEnv* env, jclass cls, int type, jobject handle) 
 {
     jthrowable exc;
-    const gchar* jtype = javatype_from_gtktype((GType)type);
-    jclass clazz = (* env)->FindClass(env, jtype);
+    const gchar* jtype;
+    jclass clazz;
+    jobject localObj;
+    jmethodID handlemid;
+    
+    jtype = javatype_from_gtktype((GType)type);
+    clazz = (* env)->FindClass(env, jtype);
     exc = (*env)->ExceptionOccurred(env);
     if (exc) {
         g_warning( "Can't find class: %s\n", jtype );
@@ -889,12 +1057,14 @@ JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_instantiateJGObjectFromGType
             return NULL;
         }
     }
-    g_free((gpointer)jtype);
     
-    jobject localObj = (* env)->AllocObject(env, clazz);
-    jmethodID handlemid = 
-        (* env)->GetMethodID(env, clazz, 
-                             "setHandle", "(Lorg/gnu/glib/Handle;)V");
+    gdk_threads_enter();
+    g_free((gpointer)jtype);
+    gdk_threads_leave();
+    
+    localObj = (* env)->AllocObject(env, clazz);
+    handlemid = (* env)->GetMethodID(env, clazz, 
+    		"setHandle", "(Lorg/gnu/glib/Handle;)V");
     exc = (*env)->ExceptionOccurred(env);
     if (exc) {
         g_warning( "No such method: setHandle(Lorg/gnu/glib/Handle;)V\n" );
@@ -907,15 +1077,21 @@ JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_instantiateJGObjectFromGType
     return localObj;
 }
 
-JNIEXPORT jint JNICALL Java_org_gnu_glib_GObject_getIntFromHandle(JNIEnv* env, jclass cls, jobject handle) 
+JNIEXPORT jint JNICALL Java_org_gnu_glib_GObject_getIntFromHandle
+    (JNIEnv* env, jclass cls, jobject handle) 
 {
-    void *handle_g = getPointerFromHandle(env, handle);
+	void *handle_g;
+	
+    handle_g = getPointerFromHandle(env, handle);
     return (jint) *(gint*)handle;
 }
 
-JNIEXPORT jstring JNICALL Java_org_gnu_glib_GObject_getStringFromHandle(JNIEnv* env, jclass cls, jobject handle) 
+JNIEXPORT jstring JNICALL Java_org_gnu_glib_GObject_getStringFromHandle
+    (JNIEnv* env, jclass cls, jobject handle) 
 {
-    void *handle_g = getPointerFromHandle(env, handle);
+	void *handle_g;
+	
+    handle_g = getPointerFromHandle(env, handle);
     return (*env)->NewStringUTF(env, (const char *)handle_g );
 }
 
@@ -923,48 +1099,46 @@ JNIEXPORT jstring JNICALL Java_org_gnu_glib_GObject_getStringFromHandle(JNIEnv*
 /*
  * My test property setting method.  :-)
  */
-JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_setTestProperty(JNIEnv* env, jclass cls, jobject object, jstring prop, jobject val) 
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_setTestProperty
+    (JNIEnv* env, jclass cls, jobject object, jstring prop, jobject val) 
 {
-    printf( "C-setTestProperty - start\n" );
-    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
-    const gchar * prop_g = 
-        (const gchar *)(*env)->GetStringUTFChars(env, prop, 0);
-    gpointer val_g = (gpointer)getPointerFromHandle(env, val);
-    printf( "C-setTestProperty - obj_g=%d,prop_g=%s,val_g=%d\n", object_g, prop_g, val_g );
+	GObject *object_g;
+	const gchar *prop_g;
+	gpointer val_g;
+	GParamSpec *pspec;
+	GValue *gval;
+	
+    object_g = (GObject *)getPointerFromHandle(env, object);
+    prop_g = (const gchar *)(*env)->GetStringUTFChars(env, prop, 0);
+    val_g = (gpointer)getPointerFromHandle(env, val);
 
-    printf( "C-setTestProperty - g_object_class_find_property(obj_g,prop_g)\n" );
-    GParamSpec *pspec = 
-        g_object_class_find_property(G_OBJECT_GET_CLASS(object_g), prop_g);
+	gdk_threads_enter();
+    pspec = g_object_class_find_property(G_OBJECT_GET_CLASS(object_g), prop_g);
     if ( !pspec ) {
+    	gdk_threads_leave();
+   		(*env)->ReleaseStringUTFChars(env, prop, prop_g);
         return;
     }
-    printf( "C-setTestProperty - got pspec\n" );
 
-    printf( "C-setTestProperty - gval = g_malloc GValue\n" );
-    GValue *gval = (GValue *) g_malloc(sizeof(GValue));
+    gval = (GValue *) g_malloc(sizeof(GValue));
     gval->g_type = 0;
-    printf( "C-setTestProperty - g_value_init( gval=%d, pspec->value_type=%d )\n", gval, pspec->value_type );
     gval = g_value_init( gval, pspec->value_type );
-    printf( "C-setTestProperty - inited\n" );
 
-    printf( "C-setTestProperty - g_value_set_ for gval=%d\n", gval );
     if ( G_VALUE_HOLDS_BOXED( gval ) ) {
-        printf( "C-setTestProperty - HOLDS_BOXED.\n" );
         g_value_set_boxed( gval, val_g );
     } else if ( G_VALUE_HOLDS_OBJECT( gval ) ) {
-        printf( "C-setTestProperty - HOLDS_OBJECT.\n" );
         g_value_set_object( gval, val_g );
     } else if ( G_VALUE_HOLDS_POINTER( gval ) ) {
-        printf( "C-setTestProperty - HOLDS_POINTER.\n" );
         g_value_set_pointer( gval, val_g );
     } else {
-        printf( "C-setTestProperty - not anything!.\n" );
+    	gdk_threads_leave();
+		(*env)->ReleaseStringUTFChars(env, prop, prop_g);
         return;
     }
-    printf( "C-setTestProperty - g_value_set completed.\n" );
-    printf( "C-setTestProperty - g_object_set_property(obj_g=%d, prop_g=%s, gval=%d).\n", object_g, prop_g, gval );
     g_object_set_property(object_g, prop_g, gval );
-    printf( "C-setTestProperty - end\n" );
+    gdk_threads_leave();
+
+	(*env)->ReleaseStringUTFChars(env, prop, prop_g);
 }
 
 #ifdef __cplusplus
