@@ -9,7 +9,8 @@
 
 #include <jni.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -29,7 +30,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Accessible_getWidget (JNIEnv *env, jc
     cptr) 
 {
     GtkAccessible *cptr_g = (GtkAccessible *)getPointerFromHandle(env, cptr);
-    return getHandleFromPointer(env, GtkAccessible_get_widget (cptr_g));
+    return getGObjectHandle(env, (GObject *) GtkAccessible_get_widget (cptr_g));
 }
 
 /*
