@@ -176,6 +176,18 @@ JNIEXPORT jstring JNICALL Java_org_gnu_gtk_Action_gtk_1action_1get_1accel_1path
 	return (*env)->NewStringUTF(env, gtk_action_get_accel_path(action_g));
 }
 
+/*
+ * Class: 	  org_gnu_gtk_Action
+ * Method:	  gtk_action_set_accel_path
+ */
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Action_gtk_1action_1set_1accel_1path
+  (JNIEnv *env, jclass cls, jobject action, jstring path)
+{
+	GtkAction* action_g = (GtkAction*)getPointerFromHandle(env, action);
+	const gchar* cpath = (*env)->GetStringUTFChars(env, path, NULL);
+	gtk_action_set_accel_path(action_g, cpath);
+}
+
 
 /*
  * Class:     org_gnu_gtk_Action
@@ -243,6 +255,18 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Action_gtk_1action_1set_1visible
 	gtk_action_set_visible(action_g, (gboolean)visible);
 }
 
+/*
+ * Class: 	  org_gnu_gtk_Action
+ * Method:	  gtk_action_set_accel_group
+ */
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Action_gtk_1action_1set_1accel_1group
+  (JNIEnv *env, jclass cls, jobject action, jobject group)
+{
+	GtkAction* action_g = (GtkAction*)getPointerFromHandle(env, action);
+	GtkAccelGroup* group_g = (GtkAccelGroup*)getPointerFromHandle(env, group);
+	gtk_action_set_accel_group(action_g, group_g);
+}
+
 #ifdef __cplusplus
 }
 #endif
