@@ -30,8 +30,8 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_Bin_gtk_1bin_1get_1type (JNIEnv *env, jc
  * Class:     org.gnu.gtk.Bin
  * Method:    gtk_bin_get_child
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Bin_gtk_1bin_1get_1child (JNIEnv *env, jclass cls, jobject 
-    bin) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Bin_gtk_1bin_1get_1child (JNIEnv *env,
+	jclass cls, jobject bin) 
 {
     GtkBin *bin_g = (GtkBin *)getPointerFromHandle(env, bin);
     return getGObjectHandle(env, (GObject *) gtk_bin_get_child (bin_g));
