@@ -9,7 +9,8 @@
 
 #include <jni.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -43,7 +44,7 @@ Java_org_gnu_gtk_CellRendererToggle_gtk_1cell_1renderer_1toggle_1get_1type (JNIE
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_CellRendererToggle_gtk_1cell_1renderer_1toggle_1new (
     JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_cell_renderer_toggle_new ());
+    return getGObjectHandle(env, (GObject *) gtk_cell_renderer_toggle_new ());
 }
 
 /*
