@@ -9,7 +9,8 @@
 
 #include <jni.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -29,7 +30,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Combo_getEntry (JNIEnv *env, jclass
 cls, jobject cptr)
 {
     GtkCombo *cptr_g = (GtkCombo *)getPointerFromHandle(env, cptr);
-    return getHandleFromPointer(env, GtkCombo_get_entry (cptr_g));
+    return getGObjectHandle(env, (GObject *) GtkCombo_get_entry (cptr_g));
 }
 
    /*
@@ -47,7 +48,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_Combo_gtk_1combo_1get_1type (JNIEnv *env
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Combo_gtk_1combo_1new (JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_combo_new ());
+    return getGObjectHandle(env, (GObject *) gtk_combo_new ());
 }
 
 /*
@@ -117,12 +118,14 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Combo_gtk_1combo_1set_1item_1string (JNI
  * Class:     org.gnu.gtk.Combo
  * Method:    gtk_combo_set_popdown_strings
  */
-JNIEXPORT void JNICALL Java_org_gnu_gtk_Combo_gtk_1combo_1set_1popdown_1strings (JNIEnv *env, 
-    jclass cls, jobject combo, jobject strings) 
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Combo_gtk_1combo_1set_1popdown_1strings
+  (JNIEnv *env, jclass cls, jobject combo, jobjectArray strings) 
 {
     GtkCombo *combo_g = (GtkCombo *)getPointerFromHandle(env, combo);
-    GList* strings_g = (GList*)getPointerFromHandle(env, strings);
+    //GList* strings_g = (GList*)getPointerFromHandle(env, strings);
+    GList* strings_g = getGListFromStringArray(env, strings);
     gtk_combo_set_popdown_strings (combo_g, strings_g);
+    releaseStringArrayInGList(env, strings, strings_g);
 }
 
 /*
