@@ -9,7 +9,8 @@
 
 #include <jni.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifndef _Included_org_gnu_gtk_FileChooserHelper
 #define _Included_org_gnu_gtk_FileChooserHelper
@@ -193,8 +194,8 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
 	len = g_slist_length(list);
 	strClass = (*env)->FindClass(env, "java/lang/String");
 	array = (*env)->NewObjectArray(env, len, strClass, NULL);
-	for (index = 0; index < len; index++) {
-		item = g_slist_nth(list, index);
+	item = list;
+	for (index = 0; item != NULL; item = item->next, ++index) {
 		value = (gchar*)item->data;
 		str = (*env)->NewStringUTF(env, value);
 		(*env)->SetObjectArrayElement(env, array, index, str);
@@ -302,8 +303,8 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
 	len = g_slist_length(list);
 	strClass = (*env)->FindClass(env, "java/lang/String");
 	array = (*env)->NewObjectArray(env, len, strClass, NULL);
-	for (index = 0; index < len; index++) {
-		item = g_slist_nth(list, index);
+	item = list;
+	for (index = 0; item != NULL; item = item->next, ++index) {
 		value = (gchar*)item->data;
 		str = (*env)->NewStringUTF(env, value);
 		(*env)->SetObjectArrayElement(env, array, index, str);
@@ -356,7 +357,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_
   (JNIEnv *env, jclass cls, jobject chooser)
 {
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-	return getHandleFromPointer(env, gtk_file_chooser_get_preview_widget(chooser_g));
+	return getGObjectHandle(env, (GObject *) gtk_file_chooser_get_preview_widget(chooser_g));
 }
 
 /*
@@ -422,7 +423,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_
   (JNIEnv *env, jclass cls, jobject chooser)
 {
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-	return getHandleFromPointer(env, gtk_file_chooser_get_extra_widget(chooser_g));
+	return getGObjectHandle(env, (GObject *) gtk_file_chooser_get_extra_widget(chooser_g));
 }
 
 /*
@@ -457,7 +458,7 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
   (JNIEnv *env, jclass cls, jobject chooser)
 {
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-	return getHandleArrayFromGSList(env, gtk_file_chooser_list_filters(chooser_g));
+	return getGObjectHandlesFromGSListAndRef(env, gtk_file_chooser_list_filters(chooser_g));
 }
 
 /*
@@ -480,47 +481,50 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_
   (JNIEnv *env, jclass cls, jobject chooser)
 {
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-	return getHandleFromPointer(env, gtk_file_chooser_get_filter(chooser_g));
+	return getGObjectHandle(env, (GObject *) gtk_file_chooser_get_filter(chooser_g));
 }
 
 /*
  * Class:     org_gnu_gtk_FileChooserHelper
  * Method:    gtk_file_chooser_add_shortcut_folder
  */
-JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1add_1shortcut_1folder
-  (JNIEnv *env, jclass cls, jobject chooser, jstring folder, jintArray error)
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1add_1shortcut_1folder
+  (JNIEnv *env, jclass cls, jobject chooser, jstring folder)
 {
-	gchar* str;
+	const gchar* str;
 	GError* err;
-	gboolean bool;
+	gboolean gbool;
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-
+	
 	str = (gchar*)(*env)->GetStringUTFChars(env, folder, NULL);
-	err = (GError*)(*env)->GetIntArrayElements(env, error, NULL);
-	bool = gtk_file_chooser_add_shortcut_folder(chooser_g, str, &err);
+	err = NULL;
+	gbool = gtk_file_chooser_add_shortcut_folder(chooser_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, folder, str);
-	(*env)->ReleaseIntArrayElements(env, error, (jint*)err, 0);
-	return (jboolean)bool;
+	if (gbool == TRUE)	{
+		return NULL;
+	}
+	return getStructHandle(env, err, NULL, (JGFreeFunc) g_error_free);
 }
 
 /*
  * Class:     org_gnu_gtk_FileChooserHelper
  * Method:    gtk_file_chooser_remove_shortcut_folder
  */
-JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1remove_1shortcut_1folder
-  (JNIEnv *env, jclass cls, jobject chooser, jstring folder, jintArray error)
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1remove_1shortcut_1folder
+  (JNIEnv *env, jclass cls, jobject chooser, jstring folder)
 {
-	gchar* str;
+	const gchar* str;
 	GError* err;
-	gboolean bool;
+	gboolean gbool;
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-
+	
 	str = (gchar*)(*env)->GetStringUTFChars(env, folder, NULL);
-	err = (GError*)(*env)->GetIntArrayElements(env, error, NULL);
-	bool = gtk_file_chooser_remove_shortcut_folder(chooser_g, str, &err);
+	gbool = gtk_file_chooser_remove_shortcut_folder(chooser_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, folder, str);
-	(*env)->ReleaseIntArrayElements(env, error, (jint*)err, 0);
-	return (jboolean)bool;
+	if (gbool == TRUE)	{
+		return NULL;
+	}
+	return getStructHandle(env, err, NULL, (JGFreeFunc) g_error_free);
 }
 
 /*
@@ -546,8 +550,8 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
 	len = g_slist_length(list);
 	strClass = (*env)->FindClass(env, "java/lang/String");
 	array = (*env)->NewObjectArray(env, len, strClass, NULL);
-	for (index = 0; index < len; index++) {
-		item = g_slist_nth(list, index);
+	item = list;
+	for (index = 0; item != NULL; item = item->next, ++index) {
 		value = (gchar*)item->data;
 		str = (*env)->NewStringUTF(env, value);
 		(*env)->SetObjectArrayElement(env, array, index, str);
@@ -559,40 +563,42 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
  * Class:     org_gnu_gtk_FileChooserHelper
  * Method:    gtk_file_chooser_add_shortcut_folder_uri
  */
-JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1add_1shortcut_1folder_1uri
-  (JNIEnv *env, jclass cls, jobject chooser, jstring folder, jintArray error)
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1add_1shortcut_1folder_1uri
+  (JNIEnv *env, jclass cls, jobject chooser, jstring folder)
 {
-	gchar* str;
+	const gchar* str;
 	GError* err;
-	gboolean bool;
+	gboolean gbool;
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-
+	
 	str = (gchar*)(*env)->GetStringUTFChars(env, folder, NULL);
-	err = (GError*)(*env)->GetIntArrayElements(env, error, NULL);
-	bool = gtk_file_chooser_add_shortcut_folder_uri(chooser_g, str, &err);
+	gbool = gtk_file_chooser_add_shortcut_folder_uri(chooser_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, folder, str);
-	(*env)->ReleaseIntArrayElements(env, error, (jint*)err, 0);
-	return (jboolean)bool;
+	if (gbool == TRUE)	{
+		return NULL;
+	}
+	return getStructHandle(env, err, NULL, (JGFreeFunc) g_error_free);
 }
 
 /*
  * Class:     org_gnu_gtk_FileChooserHelper
  * Method:    gtk_file_chooser_remove_shortcut_folder_uri
  */
-JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1remove_1shortcut_1folder_1uri
-  (JNIEnv *env, jclass cls, jobject chooser, jstring folder, jintArray error)
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1remove_1shortcut_1folder_1uri
+  (JNIEnv *env, jclass cls, jobject chooser, jstring folder)
 {
-	gchar* str;
+	const gchar* str;
 	GError* err;
-	gboolean bool;
+	gboolean gbool;
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-
+	
 	str = (gchar*)(*env)->GetStringUTFChars(env, folder, NULL);
-	err = (GError*)(*env)->GetIntArrayElements(env, error, NULL);
-	bool = gtk_file_chooser_remove_shortcut_folder_uri(chooser_g, str, &err);
+	gbool = gtk_file_chooser_remove_shortcut_folder_uri(chooser_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, folder, str);
-	(*env)->ReleaseIntArrayElements(env, error, (jint*)err, 0);
-	return (jboolean)bool;
+	if (gbool == TRUE)	{
+		return NULL;
+	}
+	return getStructHandle(env, err, NULL, (JGFreeFunc) g_error_free);
 }
 
 /*
@@ -618,8 +624,8 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
 	len = g_slist_length(list);
 	strClass = (*env)->FindClass(env, "java/lang/String");
 	array = (*env)->NewObjectArray(env, len, strClass, NULL);
-	for (index = 0; index < len; index++) {
-		item = g_slist_nth(list, index);
+	item = list;
+	for (index = 0; item != NULL; item = item->next, ++index) {
 		value = (gchar*)item->data;
 		str = (*env)->NewStringUTF(env, value);
 		(*env)->SetObjectArrayElement(env, array, index, str);
@@ -648,13 +654,6 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1ge
     return (jint)gtk_file_chooser_get_type();
 }
 
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1error_1quark
-(JNIEnv *env, jclass cls)
-{
-    GQuark quark = (GQuark)gtk_file_chooser_error_quark();
-    return getHandleFromPointer(env, (void *)&quark);
-}
-
 #ifdef __cplusplus
 }
 #endif
