@@ -1,15 +1,18 @@
 /*
  * Java-Gnome Bindings Library
  *
- * Copyright 1998-2004 the Java-Gnome Team, all rights reserved.
+ * Copyright 1998-2005 the Java-Gnome Team, all rights reserved.
  *
  * The Java-Gnome bindings library is free software distributed under
  * the terms of the GNU Library General Public License version 2.
  */
 
 #include <jni.h>
+#include <glib/gthread.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
+#include <string.h>
 
 #ifdef __cplusplus
 extern "C" 
@@ -25,10 +28,40 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Gtk_gtk_1init (JNIEnv *env,
 						   jintArray argc, 
 						   jobjectArray args) 
 {
-    gint *argc_g = (gint *) (*env)->GetIntArrayElements (env, argc, NULL);
-    gchar **args_g_arr =getStringArray(env, args);
-    gchar ***args_g = &args_g_arr;
+    gint *argc_g;
+    gchar **args_g_arr;
+    gchar ***args_g;
+
+
+    argc_g = (gint *) (*env)->GetIntArrayElements (env, argc, NULL);
+    args_g_arr = getStringArray(env, args);
+    args_g = &args_g_arr;
+
+    /*
+     * Important: glib's threads need to be enabled in order for the
+     * CustomEvents to work, but enabling gdk's threads introduces deadlocks
+     * all over the place. For so long as we can maintain the primary activity
+     * of java-gnome as single threaded GUI code, we should be alright without
+     * it. If anyone wants to enable GDK threads, then please see an archive of
+     * the java-gnome-hackers mailing list discussions circa 31 Aug 05 for
+     * discussion of the problems we encountered and some test cases to observe
+     * deadlock behaviour.
+     */
+	if (!g_thread_supported())	{
+	    g_thread_init(NULL);
+	}
+ /* gdk_threads_init(); */
+
+    /*
+     * Make sure queued calls don't get sent to GTK/GDK while we're shuting
+     * down. This is a potential trouble spot for the reasons discussed above -
+     * care will have to be taken about nesting should GTK threads be enabled.
+     * With GDK threads disabled, it's actually a no-op.
+     */
+    atexit(gdk_threads_enter);
+	
     gtk_init (argc_g, args_g);
+
     (*env)->ReleaseIntArrayElements (env, argc, (jint *) argc_g, 0);
     freeStringArray(env, args, args_g_arr);
 }
@@ -70,17 +103,19 @@ JNIEXPORT jstring JNICALL Java_org_gnu_gtk_Gtk_gtk_1set_1locale (JNIEnv *env, jc
  * Class:     org.gnu.gtk.Gtk
  * Method:    gtk_get_default_language
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Gtk_gtk_1get_1default_1language (JNIEnv *env, jclass 
-    cls) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Gtk_gtk_1get_1default_1language
+		(JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_get_default_language ());
+    return getGBoxedHandle(env, gtk_get_default_language(), PANGO_TYPE_LANGUAGE,
+    		NULL, NULL);
 }
 
 /*
  * Class:     org.gnu.gtk.Gtk
  * Method:    gtk_events_pending
  */
-JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Gtk_gtk_1events_1pending (JNIEnv *env, jclass cls) 
+JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Gtk_gtk_1events_1pending (JNIEnv
+		*env, jclass cls) 
 {
     return (jboolean)gtk_events_pending ();
 }
@@ -89,10 +124,13 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Gtk_gtk_1events_1pending (JNIEnv *en
  * Class:     org.gnu.gtk.Gtk
  * Method:    gtk_main_do_event
  */
-JNIEXPORT void JNICALL Java_org_gnu_gtk_Gtk_gtk_1main_1do_1event (JNIEnv *env, jclass cls, jobject 
-    event) 
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Gtk_gtk_1main_1do_1event (JNIEnv *env,
+		jclass cls, jobject event) 
 {
-    GdkEvent *event_g = (GdkEvent *)getPointerFromHandle(env, event);
+    GdkEvent *event_g;
+    
+    event_g = (GdkEvent *)getPointerFromHandle(env, event);
+
     gtk_main_do_event (event_g);
 }
 
@@ -149,7 +187,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Gtk_gtk_1grab_1add (JNIEnv *env, jclass
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Gtk_gtk_1grab_1get_1current (JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_grab_get_current ());
+    return getGObjectHandle(env, (GObject *) gtk_grab_get_current ());
 }
 
 /*
@@ -169,7 +207,8 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Gtk_gtk_1grab_1remove (JNIEnv *env, jcla
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Gtk_gtk_1get_1current_1event (JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_get_current_event ());
+    return getGBoxedHandle(env, gtk_get_current_event(), GDK_TYPE_EVENT,
+    		(GBoxedCopyFunc) gdk_event_copy, (GBoxedFreeFunc) gdk_event_free);
 }
 
 /*
@@ -190,7 +229,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Gtk_gtk_1get_1event_1widget (JNIEnv *
     jobject event) 
 {
     GdkEvent *event_g = (GdkEvent *)getPointerFromHandle(env, event);
-    return getHandleFromPointer(env, gtk_get_event_widget (event_g));
+    return getGObjectHandle(env, (GObject *) gtk_get_event_widget (event_g));
 }
 
 /*
@@ -204,19 +243,6 @@ JNIEXPORT jstring JNICALL Java_org_gnu_gtk_Gtk_gtk_1type_1name (JNIEnv *env, jcl
 	return (*env)->NewStringUTF(env,  (gchar*)gtk_type_name (GTK_OBJECT_TYPE(type_g)));
 }
 
-/*
- * Class:     org.gnu.gtk.Gtk
- * Method:    gtk_type_from_name
- */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Gtk_gtk_1type_1from_1name (JNIEnv *env, jclass cls, 
-    jstring name) 
-{
-    const gchar* name_g = (*env)->GetStringUTFChars( env, name, NULL);
-	jobject retval = getHandleFromPointer(env, (GType*)gtk_type_from_name (name_g));
-	(*env)->ReleaseStringUTFChars(env, name, name_g);
-	return retval;
-}
-
 JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Gtk_gtk_1true
 (JNIEnv *env, jclass cls)
 {
@@ -238,15 +264,16 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Gtk_gtk_1main_1iteration_1do
 
 static gint gtkKeySnoopFunc(GtkWidget *grab_widget, GdkEventKey *event,
                             gpointer data) {
+    jobject grab_widget_handle;
+    void *event_copy;
+    jobject event_handle;
     JGFuncCallbackRef *ref = (JGFuncCallbackRef*) data;
-    return (gint)
-        (* ref->env)->CallStaticBooleanMethod(ref->env, 
-                                              ref->cls, 
-                                              ref->methodID,
-                                              getHandleFromPointer(ref->env, 
-                                                                   grab_widget),
-                                              getHandleFromPointer(ref->env, 
-                                                                   event));
+    grab_widget_handle = getGObjectHandle(ref->env, (GObject *) grab_widget);
+    event_copy = g_malloc(sizeof(GdkEventKey));
+    memcpy(event_copy, event, sizeof(GdkEventKey));
+    event_handle = getStructHandle(ref->env, event_copy, NULL, (JGFreeFunc) g_free);
+    return (gint) (* ref->env)->CallStaticBooleanMethod(ref->env, ref->cls,
+    		ref->methodID, grab_widget_handle, event_handle);
 }
 
 JNIEXPORT jint JNICALL Java_org_gnu_gtk_Gtk_gtk_1key_1snooper_1install
