@@ -9,7 +9,8 @@
 
 #include <jni.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -33,7 +34,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_HandleBox_gtk_1handle_1box_1get_1type (J
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_HandleBox_gtk_1handle_1box_1new (JNIEnv *env, jclass 
     cls) 
 {
-    return getHandleFromPointer(env, gtk_handle_box_new ());
+    return getGObjectHandle(env, (GObject *) gtk_handle_box_new ());
 }
 
 /*
