@@ -9,7 +9,8 @@
 
 #include <jni.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -38,13 +39,12 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_MessageDialog_gtk_1message_1dialog_1n
     GtkMessageType type_g = (GtkMessageType) type;
     GtkButtonsType buttons_g = (GtkButtonsType) buttons;
     const gchar* messageFormat_g = (*env)->GetStringUTFChars(env, messageFormat, NULL);
-	jobject retval =  getHandleFromPointer(env,
+	jobject retval =  getGObjectHandle(env, (GObject *)
 			gtk_message_dialog_new (parent_g, flags_g, type_g, buttons_g, messageFormat_g));
 	(*env)->ReleaseStringUTFChars(env, messageFormat, messageFormat_g);
 	return retval;
 }
 
-
 /*
  * Class:     org_gnu_gtk_MessageDialog
  * Method:    gtk_message_dialog_new_with_markup  
@@ -54,13 +54,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_MessageDialog_gtk_1message_1dialog_1n
 {
     GtkWindow *parent_g = (GtkWindow *)getPointerFromHandle(env, parent);
 	const gchar* m = (*env)->GetStringUTFChars(env, message, NULL);
-	jobject handle = getHandleFromPointer(env,
+	jobject handle = getGObjectHandle(env, (GObject *)
 			gtk_message_dialog_new_with_markup(parent_g, (GtkDialogFlags)flags,
 					(GtkMessageType)type, (GtkButtonsType)buttons, m));
 	(*env)->ReleaseStringUTFChars(env, message, m);
 	return handle;
 }
-                                                                                
+
 /*  
  * Class:     org_gnu_gtk_MessageDialog
  * Method:    gtk_message_dialog_set_markup
@@ -73,7 +73,30 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_MessageDialog_gtk_1message_1dialog_1set_
 	gtk_message_dialog_set_markup(mdialog_g, m);
 	(*env)->ReleaseStringUTFChars(env, message, m);
 }
-                                                                                
+
+/* GTK 2.6 additions. */
+JNIEXPORT void JNICALL Java_org_gnu_gtk_MessageDialog_gtk_1message_1dialog_1format_1secondary_1text
+(JNIEnv *env, jclass cls, jobject message_dialog, jstring message_format)
+{
+    GtkMessageDialog * message_dialog_g = 
+        (GtkMessageDialog *)getPointerFromHandle(env, message_dialog);
+    const gchar * message_format_g = 
+        (const gchar *)(*env)->GetStringUTFChars(env, message_format, 0);
+    gtk_message_dialog_format_secondary_text(message_dialog_g, message_format_g);
+    (*env)->ReleaseStringUTFChars(env, message_format, message_format_g);
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_gtk_MessageDialog_gtk_1message_1dialog_1format_1secondary_1markup
+(JNIEnv *env, jclass cls, jobject message_dialog, jstring message_format)
+{
+    GtkMessageDialog * message_dialog_g = 
+        (GtkMessageDialog *)getPointerFromHandle(env, message_dialog);
+    const gchar * message_format_g = 
+        (const gchar *)(*env)->GetStringUTFChars(env, message_format, 0);
+    gtk_message_dialog_format_secondary_markup(message_dialog_g, message_format_g);
+    (*env)->ReleaseStringUTFChars(env, message_format, message_format_g);
+}
+
 #ifdef __cplusplus
 }
 
