@@ -9,7 +9,8 @@
 
 #include <jni.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -22,7 +23,11 @@ extern "C"
  */
 JNIEXPORT jint JNICALL Java_org_gnu_gtk_Plug_gtk_1plug_1get_1type (JNIEnv *env, jclass cls) 
 {
+#ifdef GTK_WINDOWING_X11
     return (jint)gtk_plug_get_type ();
+#else
+	JNU_ThrowByName(env, "java.lang.RuntimeException", "Plug: Unsupported platform");
+#endif
 }
 
 /*
@@ -32,8 +37,12 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_Plug_gtk_1plug_1get_1type (JNIEnv *env,
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Plug_gtk_1plug_1new (JNIEnv *env, jclass cls, jint 
     socketId) 
 {
+#ifdef GTK_WINDOWING_X11
     gint32 socketId_g = (gint32) socketId;
-    return getHandleFromPointer(env, gtk_plug_new (socketId_g));
+    return getGObjectHandle(env, (GObject *) gtk_plug_new (socketId_g));
+#else
+	JNU_ThrowByName(env, "java.lang.RuntimeException", "Plug: Unsupported platform");
+#endif
 }
 
 /*
@@ -43,9 +52,11 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Plug_gtk_1plug_1new (JNIEnv *env, jcl
 JNIEXPORT void JNICALL Java_org_gnu_gtk_Plug_gtk_1plug_1construct (JNIEnv *env, jclass cls, 
     jobject plug, jint socketId) 
 {
+#ifdef GTK_WINDOWING_X11
     GtkPlug *plug_g = (GtkPlug *)getPointerFromHandle(env, plug);
     gint32 socketId_g = (gint32) socketId;
     gtk_plug_construct (plug_g, socketId_g);
+#endif
 }
 
 /*
@@ -55,8 +66,12 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Plug_gtk_1plug_1construct (JNIEnv *env,
 JNIEXPORT jint JNICALL Java_org_gnu_gtk_Plug_gtk_1plug_1get_1id (JNIEnv *env, jclass cls, jobject 
     plug) 
 {
+#ifdef GTK_WINDOWING_X11
     GtkPlug *plug_g = (GtkPlug *)getPointerFromHandle(env, plug);
     return (jint)gtk_plug_get_id (plug_g);
+#else
+	JNU_ThrowByName(env, "java.lang.RuntimeException", "Plug: Unsupported platform");
+#endif
 }
 
 #ifdef __cplusplus
