@@ -10,7 +10,8 @@
 #include <jni.h>
 #include <sys/types.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -34,7 +35,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_TearoffMenuItem_gtk_1tearoff_1menu_1item
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TearoffMenuItem_gtk_1tearoff_1menu_1item_1new (JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_tearoff_menu_item_new ());
+    return getGObjectHandle(env, (GObject *) gtk_tearoff_menu_item_new ());
 }
 
 /*
