@@ -10,7 +10,8 @@
 #include <jni.h>
 #include <sys/types.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -314,7 +315,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TextTag_gtk_1text_1tag_1new
    (JNIEnv *env, jclass cls, jstring name) 
 {
     const char *utf = (*env)->GetStringUTFChars(env, name, NULL);
-    jobject handle = getHandleFromPointer(env, gtk_text_tag_new (utf));
+    jobject handle = getGObjectHandle(env, (GObject *) gtk_text_tag_new (utf));
     (*env)->ReleaseStringUTFChars(env, name, utf);
     return handle;
 }
