@@ -10,7 +10,10 @@
 #include <jni.h>
 #include <sys/types.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include <stdlib.h>
+#include <string.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -34,7 +37,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_ToolTips_gtk_1tooltips_1get_1type (JNIEn
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_ToolTips_gtk_1tooltips_1new (JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_tooltips_new ());
+    return getGObjectHandle(env, (GObject *) gtk_tooltips_new ());
 }
 
 /*
@@ -100,11 +103,16 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_ToolTips_gtk_1tooltips_1force_1window (J
  * Method:    gtk_tooltips_data_get
  * Signature: (I)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_ToolTips_gtk_1tooltips_1data_1get(JNIEnv *env, jclass cls, jobject widget)
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_ToolTips_gtk_1tooltips_1data_1get
+		(JNIEnv *env, jclass cls, jobject widget)
 {
-    GtkWidget *widget_g = 
-        (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_tooltips_data_get(widget_g));
+	GtkTooltipsData *data;
+	GtkTooltipsData *data_copy;
+    GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
+    data_copy = malloc(sizeof(GtkTooltipsData));
+    data = gtk_tooltips_data_get(widget_g);
+    memcpy(data_copy, data, sizeof(GtkTooltipsData));
+    return getStructHandle(env, data_copy, NULL, (JGFreeFunc) g_free);
 }
 
 #ifdef __cplusplus
