@@ -260,7 +260,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1column(JN
 JNIEXPORT jintArray JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1columns(JNIEnv *env, jclass cls, jobject view)
 {
     GtkTreeView *view_g = (GtkTreeView *)getPointerFromHandle(env, view);
-    return getHandleArrayFromGList(env, gtk_tree_view_get_columns(view_g));
+    return getGObjectHandlesFromGListAndRef(env, gtk_tree_view_get_columns(view_g));
 }
 
 /*
@@ -459,23 +459,35 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1set_1cursor_1o
     gboolean startEdit_g = (gboolean)startEdit;
     gtk_tree_view_set_cursor_on_cell(view_g, path_g, column_g, cell_g, startEdit_g);
 }
-                                                                          
+
 /*
  * Class:     org_gnu_gtk_TreeView
- * Method:    gtk_tree_view_get_cursor
- * Signature: (I[I[I)V
+ * Method:    gtk_tree_view_get_cursor_path
  */
-JNIEXPORT void JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1cursor(JNIEnv *env, jclass cls, jobject view, jobject path, jobject column)
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1cursor_1path
+	(JNIEnv *env, jclass cls, jobject view)
 {
-    GtkTreeView *view_g = (GtkTreeView *)getPointerFromHandle(env, view);
-    GtkTreePath *path_g = (GtkTreePath *)getPointerFromHandle(env, path);
-    GtkTreeViewColumn *column_g = 
-        (GtkTreeViewColumn *)getPointerFromHandle(env, column);
-    gtk_tree_view_get_cursor(view_g, &path_g, &column_g);
-    if (path)
-        updateHandle(env, path, path_g);
-    if (column)
-        updateHandle(env, column, column_g);
+    GtkTreeView *view_g;
+    GtkTreePath *path;
+    view_g = (GtkTreeView *)getPointerFromHandle(env, view);
+    path = NULL;
+    gtk_tree_view_get_cursor(view_g, &path, NULL);
+    return getStructHandle(env, path, NULL, (JGFreeFunc) gtk_tree_path_free);
+}
+
+/*
+ * Class:     org_gnu_gtk_TreeView
+ * Method:    gtk_tree_view_get_cursor_column
+ */
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1cursor_1column
+	(JNIEnv *env, jclass cls, jobject view)
+{
+    GtkTreeView *view_g;
+    GtkTreeViewColumn *column;
+    view_g = (GtkTreeView *) getPointerFromHandle(env, view);
+    column = NULL; 
+    gtk_tree_view_get_cursor(view_g, NULL, &column);
+    return getGObjectHandle(env, (GObject *) column);
 }
                                                                           
 
@@ -790,7 +802,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1set_1row_1sepa
     ref->methodID = 
         (*env)->GetMethodID(env, 
                             (*env)->GetObjectClass(env, ref->obj), 
-                            funcname, "(Lorg/gnu/javagnome/Handle;Lorg/gnu/javagnome/Handle;)Z" );
+                            funcname, "(Lorg/gnu/glib/Handle;Lorg/gnu/glib/Handle;)Z" );
     if ( ref->methodID == NULL ) {
         (*env)->ReleaseStringUTFChars(env, callback, funcname);
         g_free( ref );
