@@ -11,7 +11,8 @@
 #include <sys/types.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -28,28 +29,6 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1type (JNIEnv *e
     return (jint)gtk_widget_get_type ();
 }
 
-/*
- * Class:     org.gnu.gtk.Widget
- * Method:    gtk_widget_ref
- * Signature: (I)I
- */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1ref (JNIEnv *env, jclass cls, jobject widget) 
-{
-    GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_ref (widget_g));
-}
-
-/*
- * Class:     org.gnu.gtk.Widget
- * Method:    gtk_widget_unref
- * Signature: (I)V
- */
-JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1unref (JNIEnv *env, jclass cls, jobject widget) 
-{
-    GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    gtk_widget_unref (widget_g);
-}
-
 /*
  * Class:     org.gnu.gtk.Widget
  * Method:    gtk_widget_destroy
@@ -281,7 +260,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1size_1allocate (JNIE
  * Method:    gtk_widget_add_accelerator
  * Signature: (I[BIIII)V
  */
-JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1add_1accelerator (JNIEnv *env, jclass cls, jobject widget, jstring accelSignal, jobject accelGroup, jint accelKey, jint accelMods, jint accelFlags) 
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1add_1accelerator (JNIEnv *env, jclass cls, jobject widget, jobject accelGroup, jint accelKey, jint accelMods, jint accelFlags) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     GtkAccelGroup *accelGroup_g = 
@@ -289,11 +268,8 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1add_1accelerator (JN
     guint32 accelKey_g = (guint32) accelKey;
     guint32 accelMods_g = (guint32) accelMods;
     GtkAccelFlags accelFlags_g = (GtkAccelFlags) accelFlags;
-    const gchar* accelSignal_g = 
-        (*env)->GetStringUTFChars(env, accelSignal, NULL);
-    gtk_widget_add_accelerator (widget_g, accelSignal_g, accelGroup_g, 
+    gtk_widget_add_accelerator (widget_g, "activate", accelGroup_g, 
                                 accelKey_g, accelMods_g, accelFlags_g);
-    (*env)->ReleaseStringUTFChars(env, accelSignal, accelSignal_g);
 }
 
 /*
@@ -425,16 +401,19 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1reparent (JNIEnv *en
  * Method:    gtk_widget_intersect
  * Signature: (II[Lint ;)Z
  */
-JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1intersect (JNIEnv *env, jclass cls, jobject widget, jobject area, jobject intersection) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1intersect
+		(JNIEnv *env, jclass cls, jobject widget, jobject area) 
 {
-    // this might be buggy!?!
-
-    GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    GdkRectangle *area_g = (GdkRectangle *)getPointerFromHandle(env, area);
-    GdkRectangle *intersection_g = g_new(GdkRectangle, 1);
-    jboolean result_j = (jboolean) (gtk_widget_intersect (widget_g, area_g, intersection_g));
-    updateHandle(env, intersection, intersection_g);
-    return result_j;
+    GtkWidget *widget_g = (GtkWidget *) getPointerFromHandle(env, widget);
+    GdkRectangle *area_g = (GdkRectangle *) getPointerFromHandle(env, area);
+    GdkRectangle *intersection = g_new(GdkRectangle, 1);
+    gboolean result = gtk_widget_intersect (widget_g, area_g, intersection);
+    if (result == TRUE)	{
+    	return getGBoxedHandle(env, intersection, GDK_TYPE_RECTANGLE, NULL,
+    		(GBoxedFreeFunc) g_free);
+    }
+    g_free(intersection);
+    return NULL;
 }
 
 /*
@@ -660,7 +639,7 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1child_1visi
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1parent (JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_parent (widget_g));
+    return getGObjectHandle(env, (GObject *) gtk_widget_get_parent (widget_g));
 }
 
 /*
@@ -671,7 +650,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1parent (JNIE
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1parent_1window (JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_parent_window (widget_g));
+    return getGObjectHandle(env, (GObject *) gtk_widget_get_parent_window (widget_g));
 }
 
 /*
@@ -682,9 +661,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1parent_1wind
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1window (JNIEnv *env, jclass cls, jobject widget)
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, 
-                                (GTK_WIDGET_NO_WINDOW(widget_g) ? 
-                                 NULL : widget_g->window));
+    GObject *obj = (GObject *) (GTK_WIDGET_NO_WINDOW(widget_g) ? NULL : widget_g->window);
+    return getGObjectHandle(env, obj);
 }
 
 /*
@@ -782,7 +760,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1extension_1even
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1toplevel (JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_toplevel (widget_g));
+    return getGObjectHandle(env, (GObject *) gtk_widget_get_toplevel (widget_g));
 }
 
 /*
@@ -794,7 +772,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1ancestor(JNI
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     GType widgetType_g = (GType)widgetType;
-    return getHandleFromPointer(env, gtk_widget_get_ancestor(widget_g, widgetType_g));
+    return getGObjectHandle(env, (GObject *) gtk_widget_get_ancestor(widget_g, widgetType_g));
 }
 
 /*
@@ -805,7 +783,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1ancestor(JNI
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1colormap (JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_colormap (widget_g));
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_colormap (widget_g));
 }
 
 /*
@@ -816,7 +794,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1colormap (JN
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1visual (JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_visual (widget_g));
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_visual (widget_g));
 }
 
 /*
@@ -824,10 +802,11 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1visual (JNIE
  * Method:    gtk_widget_get_screen
  * Signature: (I)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1screen(JNIEnv *env, jclass cls, jobject widget)
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1screen
+		(JNIEnv *env, jclass cls, jobject widget)
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_screen(widget_g));
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_screen(widget_g));
 }
 
 /*
@@ -835,7 +814,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1screen(JNIEn
  * Method:    gtk_widget_has_screen
  * Signature: (I)Z
  */
-JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1has_1screen(JNIEnv *env, jclass cls, jobject widget)
+JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1has_1screen
+		(JNIEnv *env, jclass cls, jobject widget)
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     return (jboolean)gtk_widget_has_screen(widget_g);
@@ -846,10 +826,11 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1has_1screen(JNIE
  * Method:    gtk_widget_get_display
  * Signature: (I)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1display(JNIEnv *env, jclass cls, jobject widget)
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1display
+		(JNIEnv *env, jclass cls, jobject widget)
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_display(widget_g));
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_display(widget_g));
 }
 
 /*
@@ -860,7 +841,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1display(JNIE
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1root_1window(JNIEnv *env, jclass cls, jobject widget)
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_root_window(widget_g));
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_root_window(widget_g));
 }
 
 /*
@@ -868,7 +849,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1root_1window
  * Method:    gtk_widget_get_sensitive
  * Signature: (I)I
  */
-JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1sensitive( JNIEnv *env, jclass cls, jobject widget )
+JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1sensitive
+		( JNIEnv *env, jclass cls, jobject widget )
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     return (jboolean) (GTK_WIDGET_SENSITIVE(widget_g));
@@ -879,10 +861,11 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1sensitive(
  * Method:    gtk_widget_get_settings
  * Signature: (I)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1settings (JNIEnv *env, jclass cls, jobject widget) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1settings
+		(JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_settings (widget_g));
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_settings (widget_g));
 }
 
 /*
@@ -890,10 +873,11 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1settings (JN
  * Method:    gtk_widget_get_accessible
  * Signature: (I)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1accessible (JNIEnv *env, jclass cls, jobject widget) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1accessible
+		(JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_accessible (widget_g));
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_accessible (widget_g));
 }
 
 /*
@@ -987,7 +971,8 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1hide_1on_1delete
  * Method:    gtk_widget_set_style
  * Signature: (II)V
  */
-JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1set_1style (JNIEnv *env, jclass cls, jobject widget, jobject style) 
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1set_1style
+		(JNIEnv *env, jclass cls, jobject widget, jobject style) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     GtkStyle *style_g = (GtkStyle *)getPointerFromHandle(env, style);
@@ -1010,10 +995,11 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1ensure_1style (JNIEn
  * Method:    gtk_widget_get_style
  * Signature: (I)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1style (JNIEnv *env, jclass cls, jobject widget) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1style
+		(JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_style (widget_g));
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_style (widget_g));
 }
 
 /*
@@ -1021,7 +1007,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1style (JNIEn
  * Method:    gtk_widget_modify_style
  * Signature: (II)V
  */
-JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1modify_1style (JNIEnv *env, jclass cls, jobject widget, jobject style) 
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1modify_1style
+		(JNIEnv *env, jclass cls, jobject widget, jobject style) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     GtkRcStyle *style_g = (GtkRcStyle *)getPointerFromHandle(env, style);
@@ -1033,10 +1020,11 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1modify_1style (JNIEn
  * Method:    gtk_widget_get_modifier_style
  * Signature: (I)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1modifier_1style (JNIEnv *env, jclass cls, jobject widget) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1modifier_1style
+		(JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_modifier_style (widget_g));
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_modifier_style (widget_g));
 }
 
 /*
@@ -1109,10 +1097,11 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1modify_1font (JNIEnv
  * Method:    gtk_widget_create_pango_context
  * Signature: (I)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1create_1pango_1context (JNIEnv *env, jclass cls, jobject widget) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1create_1pango_1context
+		(JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_create_pango_context (widget_g));
+    return getGObjectHandle(env, (GObject *) gtk_widget_create_pango_context (widget_g));
 }
 
 /*
@@ -1120,10 +1109,11 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1create_1pango_1co
  * Method:    gtk_widget_get_pango_context
  * Signature: (I)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1pango_1context (JNIEnv *env, jclass cls, jobject widget) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1pango_1context
+		(JNIEnv *env, jclass cls, jobject widget) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleFromPointer(env, gtk_widget_get_pango_context (widget_g));
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_pango_context (widget_g));
 }
 
 /*
@@ -1131,13 +1121,15 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1pango_1conte
  * Method:    gtk_widget_create_pango_layout
  * Signature: (I[B)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1create_1pango_1layout (JNIEnv *env, jclass cls, jobject widget, jstring text) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1create_1pango_1layout
+		(JNIEnv *env, jclass cls, jobject widget, jstring text) 
 {
+	PangoLayout *layout;
+	jobject retval;
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     const gchar* text_g = (*env)->GetStringUTFChars(env, text, NULL);
-    jobject retval = 
-        getHandleFromPointer(env, gtk_widget_create_pango_layout (widget_g, 
-                                                                  text_g));
+    layout = gtk_widget_create_pango_layout (widget_g, text_g);
+    retval = getGObjectHandle(env, (GObject *) layout);
     (*env)->ReleaseStringUTFChars(env, text, text_g);
     return retval;
 }
@@ -1154,8 +1146,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1render_1icon (JNI
     const gchar* stockID_g = (*env)->GetStringUTFChars(env, stockID, NULL);
     const gchar* detail_g = (*env)->GetStringUTFChars(env, detail, NULL);
     jobject retval = 
-        getHandleFromPointer(env, gtk_widget_render_icon (widget_g, stockID_g, 
-                                                          size_g, detail_g));
+        getGObjectHandleAndRef(env, (GObject *) gtk_widget_render_icon (widget_g,
+        		stockID_g, size_g, detail_g));
     (*env)->ReleaseStringUTFChars( env, stockID, stockID_g );
     (*env)->ReleaseStringUTFChars( env, detail, detail_g );
     return retval;
@@ -1267,9 +1259,10 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1set_1default_1colorm
  * Method:    gtk_widget_get_default_colormap
  * Signature: ()I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1default_1colormap (JNIEnv *env, jclass cls) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1default_1colormap
+		(JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_widget_get_default_colormap ());
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_default_colormap ());
 }
 
 /*
@@ -1277,9 +1270,10 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1default_1col
  * Method:    gtk_widget_get_default_visual
  * Signature: ()I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1default_1visual (JNIEnv *env, jclass cls) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1default_1visual
+		(JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_widget_get_default_visual ());
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_default_visual ());
 }
 
 /*
@@ -1287,9 +1281,10 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1default_1vis
  * Method:    gtk_widget_get_default_style
  * Signature: ()I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1default_1style (JNIEnv *env, jclass cls) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1default_1style
+		(JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_widget_get_default_style ());
+    return getGObjectHandleAndRef(env, (GObject *) gtk_widget_get_default_style ());
 }
 
 /*
@@ -1297,7 +1292,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1get_1default_1sty
  * Method:    gtk_widget_set_direction
  * Signature: (II)V
  */
-JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1set_1direction (JNIEnv *env, jclass cls, jobject widget, jint dir) 
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1set_1direction
+		(JNIEnv *env, jclass cls, jobject widget, jint dir) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     GtkTextDirection dir_g = (GtkTextDirection) dir;
@@ -1358,7 +1354,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1shape_1combine_1mask
 JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1path (JNIEnv *env, jclass cls, jobject widget, jintArray pathLength, jobjectArray path, jobjectArray pathReversed) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    gint *pathLength_g = (gint *) (*env)->GetIntArrayElements (env, pathLength, NULL);
+    guint *pathLength_g = (guint *) (*env)->GetIntArrayElements (env, pathLength, NULL);
     gchar **path_g = getStringArray(env, path);
     gchar **pathReversed_g = getStringArray(env, pathReversed);
     gtk_widget_path (widget_g, pathLength_g, path_g, pathReversed_g);
@@ -1375,7 +1371,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1path (JNIEnv *env, j
 JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1class_1path (JNIEnv *env, jclass cls, jobject widget, jintArray pathLength, jobjectArray path, jobjectArray pathReversed) 
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    gint *pathLength_g = (gint *) (*env)->GetIntArrayElements (env, pathLength, NULL);
+    guint *pathLength_g = (guint *) (*env)->GetIntArrayElements (env, pathLength, NULL);
     gchar **path_g = getStringArray(env, path);
     gchar **pathReversed_g = getStringArray(env, pathReversed);
     gtk_widget_class_path (widget_g, pathLength_g, path_g, pathReversed_g);
@@ -1392,7 +1388,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1class_1path (JNIEnv
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_Widget_gtk_1widget_1list_1mnemonic_1labels(JNIEnv *env, jclass cls, jobject widget)
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
-    return getHandleArrayFromGList(env, gtk_widget_list_mnemonic_labels(widget_g));
+    return getGObjectHandlesFromGList(env, gtk_widget_list_mnemonic_labels(widget_g));
 }
 
 /*
@@ -1433,20 +1429,10 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1drag_1source_1set(JNIEnv *en
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     GdkModifierType buttonMask_g = (GdkModifierType)buttonMask;
     GdkDragAction actions_g = (GdkDragAction)actions;
-
-    GtkTargetEntry** entries;
-    jsize len;
-    int index;
-    len = (*env)->GetArrayLength(env, targets);
-    entries = g_malloc(sizeof(GtkTargetEntry*)*len);
-    for (index = 0; index < len; index++)
-        entries[index] = 
-            (GtkTargetEntry*)
-            getPointerFromHandle(env, (*env)->GetObjectArrayElement(env, 
-                                                                    targets, 
-                                                                    index));
-
-    gtk_drag_source_set(widget_g, buttonMask_g, entries[0], len, actions_g);
+    const GtkTargetEntry* entries = (GtkTargetEntry *)getArrayFromHandles(env,
+    	targets, sizeof(GtkTargetEntry), JNI_FALSE, JNI_FALSE);
+    jsize len = (*env)->GetArrayLength(env, targets);
+    gtk_drag_source_set(widget_g, buttonMask_g, entries, (gint) len, actions_g);
 }
 
 /*
@@ -1505,13 +1491,15 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1drag_1source_1set_1icon_1sto
  * Method:    gtk_drag_get_data
  * Signature: (III)V
  */
-JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1drag_1get_1data(JNIEnv *env, jclass cls, jobject widget, jobject context, jint target)
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1drag_1get_1data(JNIEnv *env, jclass cls, jobject widget, jobject context, jobject target, jint time)
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     GdkDragContext *context_g = 
         (GdkDragContext *)getPointerFromHandle(env, context);
-    GdkAtom target_g = (GdkAtom)target;
-    gtk_drag_get_data(widget_g, context_g, target_g, GDK_CURRENT_TIME);
+    
+    GdkAtom target_g = getPointerFromHandle(env, target);
+    //(GdkAtom)target;
+    gtk_drag_get_data(widget_g, context_g, target_g, time);
 }
 
 /*
@@ -1519,13 +1507,13 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1drag_1get_1data(JNIEnv *env,
  * Method:    gtk_drag_finish
  * Signature: (IZZ)V
  */
-JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1drag_1finish(JNIEnv *env, jclass cls, jobject context, jboolean success, jboolean del)
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1drag_1finish(JNIEnv *env, jclass cls, jobject context, jboolean success, jboolean del, jint time)
 {
     GdkDragContext *context_g = 
         (GdkDragContext *)getPointerFromHandle(env, context);
     gboolean success_g = (gboolean)success;
     gboolean del_g = (gboolean)del;
-    gtk_drag_finish(context_g, success_g, del_g, GDK_CURRENT_TIME);
+    gtk_drag_finish(context_g, success_g, del_g, time);
 }
 
 /*
@@ -1559,10 +1547,10 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Widget_gtk_1drag_1dest_1set(JNIEnv *env,
 {
     GtkWidget *widget_g = (GtkWidget *)getPointerFromHandle(env, widget);
     GtkDestDefaults flags_g = (GtkDestDefaults)flags;
-    GdkDragAction actions_g = (GdkDragAction)actions;
-    GtkTargetEntry* entries = (GtkTargetEntry*)getPointerArrayFromHandles(env, targets);
+    const GdkDragAction actions_g = (GdkDragAction)actions;
+    const GtkTargetEntry* entries = (GtkTargetEntry *)getArrayFromHandles(env, targets, sizeof(GtkTargetEntry), JNI_FALSE, JNI_FALSE);
     jsize len = (*env)->GetArrayLength(env, targets);
-    gtk_drag_dest_set(widget_g, flags_g, entries, len, actions_g);
+    gtk_drag_dest_set(widget_g, flags_g, entries, (gint)len, actions_g);
 }
 
 /*
