@@ -9,7 +9,7 @@
 
 #include <jni.h>
 #include <pango/pango.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
 
 #ifdef __cplusplus
 extern "C" 
@@ -20,8 +20,8 @@ extern "C"
  * Class:     org.gnu.pango.Attribute
  * Method:    getStartIndex
  */
-JNIEXPORT jint JNICALL Java_org_gnu_pango_Attribute_getStartIndex (JNIEnv *env, jclass cls, 
-    jobject obj) 
+JNIEXPORT jint JNICALL Java_org_gnu_pango_Attribute_getStartIndex 
+  (JNIEnv *env, jclass cls, jobject obj) 
 {
     PangoAttribute *obj_g = (PangoAttribute *)getPointerFromHandle(env, obj);
     return (jint)obj_g->start_index;
@@ -31,14 +31,13 @@ JNIEXPORT jint JNICALL Java_org_gnu_pango_Attribute_getStartIndex (JNIEnv *env,
  * Class:     org.gnu.pango.Attribute
  * Method:    getEndIndex
  */
-JNIEXPORT jint JNICALL Java_org_gnu_pango_Attribute_getEndIndex (JNIEnv *env, jclass cls, jobject 
-    obj) 
+JNIEXPORT jint JNICALL Java_org_gnu_pango_Attribute_getEndIndex 
+  (JNIEnv *env, jclass cls, jobject obj) 
 {
     PangoAttribute *obj_g = (PangoAttribute *)getPointerFromHandle(env, obj);
     return (jint)obj_g->end_index;
 }
 
-
 #ifdef __cplusplus
 }
 
